"""
Compute squared SRC indices confidence intervals
------------------------------------------------
"""

# %%
# This example shows how to compute squared SRC indices confidence bounds with bootstrap.
# First, we compute squared SRC indices and draw them.
# Then we compute bootstrap confidence bounds using the :class:`~openturns.BootstrapExperiment` class and draw them.

# %%
# Define the model
# ~~~~~~~~~~~~~~~~

# %%
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import flood_model

# %%
# Load the flood model.
fm = flood_model.FloodModel()
distribution = fm.distribution
g = fm.model.getMarginal(1)
dim = distribution.getDimension()

# %%
# See the distribution
distribution

# %%
# See the model
g.getOutputDescription()

# %%
# Estimate the squared SRC indices
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# %%
# We produce a pair of input and output sample.
N = 100
X = distribution.getSample(N)
Y = g(X)

# %%
# Compute squared SRC indices from the generated design.
importance_factors = ot.CorrelationAnalysis(X, Y).computeSquaredSRC()
print(importance_factors)

# %%
# Plot the squared SRC indices.
input_names = g.getInputDescription()
graph = ot.SobolIndicesAlgorithm.DrawCorrelationCoefficients(
    importance_factors, input_names, "Importance factors"
)
graph.setYTitle("Squared SRC")
_ = otv.View(graph)

# %%
# Compute confidence intervals
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# %%
# We now compute bootstrap confidence intervals for the importance factors.
# This is done with the :class:`~openturns.BootstrapExperiment` class.

# %%
# Create SRC bootstrap sample
bootstrap_size = 100
src_boot = ot.Sample(bootstrap_size, dim)
for i in range(bootstrap_size):
    selection = ot.BootstrapExperiment.GenerateSelection(N, N)
    X_boot = X[selection]
    Y_boot = Y[selection]
    src_boot[i, :] = ot.CorrelationAnalysis(X_boot, Y_boot).computeSquaredSRC()

# %%
# Compute bootstrap quantiles
alpha = 0.05
src_lb = src_boot.computeQuantilePerComponent(alpha / 2.0)
src_ub = src_boot.computeQuantilePerComponent(1.0 - alpha / 2.0)
src_interval = ot.Interval(src_lb, src_ub)
print(src_interval)


# %%
def draw_importance_factors_with_bounds(
    importance_factors, input_names, alpha, importance_bounds
):
    """
    Plot importance factors indices with confidence bounds of level 1 - alpha.

    Parameters
    ----------
    importance_factors : Point(dimension)
        The importance factors.
    input_names : list(str)
        The names of the input variables.
    alpha : float, in [0, 1]
        The complementary confidence level.
    importance_bounds : Interval(dimension)
        The lower and upper bounds of the importance factors

    Returns
    -------
    graph : Graph
        The importance factors indices with lower and upper 1-alpha confidence intervals.
    """
    dim = importance_factors.getDimension()
    lb = importance_bounds.getLowerBound()
    ub = importance_bounds.getUpperBound()
    palette = ot.Drawable.BuildDefaultPalette(2)
    graph = ot.SobolIndicesAlgorithm.DrawCorrelationCoefficients(
        importance_factors, input_names, "Importance factors"
    )
    graph.setColors([palette[0], "black"])
    graph.setYTitle("Squared SRC")

    # Add confidence bounds
    for i in range(dim):
        curve = ot.Curve([1 + i, 1 + i], [lb[i], ub[i]])
        curve.setLineWidth(2.0)
        curve.setColor(palette[1])
        graph.add(curve)
    return graph


# %%
# sphinx_gallery_thumbnail_number = 2

# %%
# Plot the SRC indices mean and confidence intervals.
src_mean = src_boot.computeMean()
graph = draw_importance_factors_with_bounds(src_mean, input_names, alpha, src_interval)
graph.setTitle(f"Importance factors - CI {(1.0 - alpha) * 100:.2f}%")
_ = otv.View(graph)

# %%
# We see that the variable :math:`Q` must be significant, because the lower
# bound of the confidence interval does not cross the `X` axis.
# Furthermore, its bounds are significantly greater than the bounds of the
# other variables (although perhaps less significantly for the :math:`K_s` variable).
# Hence, it must be recognized that :math:`Q` is the most important variable
# in this model, according to the linear regression model.
#
# We see that the variable :math:`Z_m` has an importance factor close to zero,
# taking into account the confidence bounds (which are very small in this case).
# Hence, the variable :math:`Z_m` could be replaced by a constant without
# reducing the variance of the output much.
#
# The variables :math:`K_s`, :math:`Z_v` and :math:`H_d` are somewhat in-between these two
# extreme situations. We cannot state that one of them is of greater importance
# than the other, because the confidence bounds are of comparable magnitude.
# Looking only at the importance factors, we may wrongly conclude that
# :math:`Z_v` has a greater impact than :math:`K_s` because the estimate of the
# importance factor for :math:`Z_v` is strictly greater than the estimate for
# :math:`K_s`. But taking into account for the variability of the estimators,
# this conclusion has no foundation, since confidence limits are comparable.
# In order to distinguish between the impact of these two variables, a larger sample size is needed.

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import math
import openturns.testing as ott

ot.TESTPREAMBLE()

f = ot.SymbolicFunction(["x"], ["sin(x)"])
a = -2.5
b = 4.5
# Default parameters
algo = ot.GaussLegendre()
print("Algo=", algo)
# High-level interface
value, adaptedNodes = algo.integrateWithNodes(f, ot.Interval(a, b))
ref = math.cos(a) - math.cos(b)
print("value=%.6f" % value[0], ", ref=%.6f" % ref, ", adaptedNodes=", adaptedNodes)
# Low-level interface
algo = ot.GaussLegendre([20])
value, adaptedNodes = algo.integrateWithNodes(f, ot.Interval(a, b))
print("value=%.6f" % value[0], ", ref=%.6f" % ref, ", adaptedNodes=", adaptedNodes)


# Check polynomial degree of exactness
def checkPolynomialExactness(
    marginalDegrees,
    marginalSizes,
    lowerBound=0.0,
    upperBound=1.0,
    rtol=1.0e-15,
    atol=0.0,
    verbose=False,
):
    """
    Check polynomial exactness of GaussLegendre quadrature

    Parameters
    ----------
    marginalDegrees : list of int
        The polynomial degree of the marginal polynomials to integrate
    marginalSizes : list of int
        The number of nodes on each marginal axis.
    lowerBound : float
        The lower bound of quadrature
    upperBound : float
        The upper bound of quadrature
    rtol : float, > 0
        The relative tolerance
    atol : float, > 0
        The absolute tolerance
    verbose : bool
        Set to True to print intermediate messages

    Examples
    --------
    marginalDegrees = [5, 3, 7]
    marginalSizes = [3, 2, 4]  # Polynomial exactness space = P5 x P3 x P7
    checkPolynomialExactness(marginalDegrees, marginalSizes)
    """
    dimension = len(marginalDegrees)
    if len(marginalSizes) != dimension:
        raise ValueError(
            f"Number of marginal degrees is {dimension} "
            f"but number of marginal sizes is {len(marginalSizes)}"
        )

    # Set bounds
    bounds = ot.Interval([lowerBound] * dimension, [upperBound] * dimension)

    # Create polynomial
    polynomialCollection = ot.PolynomialCollection()
    for i in range(dimension):
        coefficients = [0.0] * (1 + marginalDegrees[i])
        coefficients[-1] = 1
        polynomial = ot.UniVariatePolynomial(coefficients)
        polynomialCollection.add(polynomial)

    productPoly = ot.ProductPolynomialEvaluation(polynomialCollection)

    # Evaluate integral
    algo = ot.GaussLegendre(marginalSizes)
    computedIntegral = algo.integrate(ot.Function(productPoly), bounds)[0]

    # Expected integral
    lowerBoundPoint = bounds.getLowerBound()
    upperBoundPoint = bounds.getUpperBound()
    expectedIntegral = 1.0
    for i in range(dimension):
        marginalIntegral = (
            upperBoundPoint[i] ** (1 + marginalDegrees[i])
            - lowerBoundPoint[i] ** (1 + marginalDegrees[i])
        ) / (1 + marginalDegrees[i])
        expectedIntegral *= marginalIntegral
    absoluteError = abs(computedIntegral - expectedIntegral)
    if verbose:
        print(
            f"Polynomial : {str(productPoly):20s}, "
            f"  integral computed  = {computedIntegral:.7f}, "
            f"  expected = {expectedIntegral:.7f}, "
            f"  absolute error = {absoluteError:.3e}"
        )
    ott.assert_almost_equal(expectedIntegral, computedIntegral, rtol, atol)


# Test different polynomials, up to the maximum
marginalSizes = [3, 2, 4]  # Polynomial exactness space = P5 x P3 x P7
maximumMarginalDegrees = [2 * n for n in marginalSizes]
experiment = ot.Tuples(maximumMarginalDegrees)
marginalDegreesList = experiment.generate()
for i in range(marginalDegreesList.getSize()):
    marginalDegrees = marginalDegreesList[i]
    checkPolynomialExactness(
        marginalDegrees, marginalSizes, rtol=1.0e-14, verbose=False
    )



import openturns as ot
import openturns.viewer as otv

grid = ot.GridLayout(1, 2)
pdf_2d = ot.Graph("KPermutations, PDF", "x1", "x2")
cdf_2d = ot.Graph("KPermutations, CDF", "x1", "x2")

distribution_2d = ot.KPermutationsDistribution(2, 10)

xMin = [-0.1] * 2
xMax = [10.1] * 2

pdf_2d.add(distribution_2d.drawPDF(xMin, xMax))
pdf_2d.setLegends([""])
cdf_2d.add(distribution_2d.drawCDF(xMin, xMax, [101] * 2))
cdf_2d.setLegends([""])
grid.setGraph(0, 0, pdf_2d)
grid.setGraph(0, 1, cdf_2d)
grid.setTitle("KPermutationsDistribution")
grid.setLegendPosition("upper right")
v = otv.View(grid)
fig = v.getFigure()
fig.axes[1].legend(loc="best")



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
from math import exp, inf

ot.TESTPREAMBLE()

ot.RandomGenerator.SetSeed(0)

mu = 5000.0
prior = ot.Normal(mu, 1.0)
initialState = [0.0]
instrumental = ot.Normal(0.0, 1.0)
sampler = ot.RandomWalkMetropolisHastings(prior, initialState, instrumental)
sampler.setBurnIn(1000)
s1 = sampler.getSample(2000)
ott.assert_almost_equal(s1[sampler.getBurnIn() :].computeMean()[0], mu, 1e-2, 0.0)

data = ot.Sample(
    [
        [53, 1],
        [57, 1],
        [58, 1],
        [63, 1],
        [66, 0],
        [67, 0],
        [67, 0],
        [67, 0],
        [68, 0],
        [69, 0],
        [70, 0],
        [70, 0],
        [70, 1],
        [70, 1],
        [72, 0],
        [73, 0],
        [75, 0],
        [75, 1],
        [76, 0],
        [76, 0],
        [78, 0],
        [79, 0],
        [81, 0],
    ]
)

data.setDescription(["Temp. (°F)", "Failure"])
print(data)


fun = ot.SymbolicFunction(
    ["alpha", "beta", "x"], ["exp(alpha + beta * x) / (1 + exp(alpha + beta * x))"]
)
linkFunction = ot.ParametricFunction(fun, [2], [0.0])
instrumental = ot.Normal([0.0] * 2, [0.5, 0.05], ot.IdentityMatrix(2))

target = ot.JointDistribution([ot.Uniform(-100.0, 100.0)] * 2)
rwmh = ot.RandomWalkMetropolisHastings(target, [0.0] * 2, instrumental)
rwmh.setBurnIn(10000)
conditional = ot.Bernoulli()
observations = data[:, 1]
covariates = data[:, 0]
rwmh.setLikelihood(conditional, observations, linkFunction, covariates)

# try to generate a sample
sample = rwmh.getSample(100000)
mu = sample[rwmh.getBurnIn() :].computeMean()
sigma = sample[rwmh.getBurnIn() :].computeStandardDeviation()
print("mu=", mu, "sigma=", sigma)


# Put the posterior density in a PythonFunction
def post_den(alpha_beta):
    return [exp(rwmh.computeLogPosterior(alpha_beta))]


posterior_density = ot.PythonFunction(2, 1, post_den)

# approximate the posterior distribution
# NB: to get a good view of the mode of the posterior distribution, use:
# posterior_density.draw([14.0, -0.25], [16.0, -0.22], [100, 100])
mesher = ot.IntervalMesher([100, 100])
lowerBound = [-2.5, -0.53]
upperBound = [34.0, 0.03]
box = ot.Interval(lowerBound, upperBound)
vertices = mesher.build(box).getVertices()
densities = posterior_density(vertices).asPoint()
approximate_posterior = ot.FiniteDiscreteDistribution(vertices, densities)
approximate_mean = approximate_posterior.getMean()
approximate_std = approximate_posterior.getStandardDeviation()

ott.assert_almost_equal(mu, approximate_mean, 0.2, 0.0)
ott.assert_almost_equal(sigma, approximate_std, 0.2, 0.0)

print("acceptance rate=", rwmh.getAcceptanceRate())
ott.assert_almost_equal(
    rwmh.getAcceptanceRate(), 0.28, 0.1, 0.0
)  # Empirical acceptance rate observed when executing the code

# from 1532
fullModel = ot.SymbolicFunction(["x", "theta"], ["theta", "1.0"])
model = ot.ParametricFunction(fullModel, [0], [1.0])
prior = ot.Normal(0.0, 1.0)
prior.setDescription(["theta"])
instrumental = ot.Normal(0.0, 1.0)
thetaTrue = [2.0]
# We choose the most favorable initial state: the true parameter value.
initialState = thetaTrue
conditional = ot.Normal()  # the log-likelihood is Gaussian
# 503 observations: OK
obsSize = 503
ot.RandomGenerator.SetSeed(0)
y_obs = ot.Normal(thetaTrue[0], 1.0).getSample(obsSize)
x_obs = y_obs
RWMHsampler = ot.RandomWalkMetropolisHastings(prior, initialState, instrumental)
RWMHsampler.setLikelihood(conditional, y_obs, model, x_obs)
print(
    "Log-likelihood of thetaTrue = {!r}".format(
        RWMHsampler.computeLogLikelihood(thetaTrue)
    )
)
real_503 = RWMHsampler.getRealization()
print("With 503 observations, getRealization() produces {!r}".format(real_503[0]))
ott.assert_almost_equal(real_503[0], 2.0)
# 504 observations: not OK
obsSize = 504
ot.RandomGenerator.SetSeed(0)
y_obs = ot.Normal(thetaTrue[0], 1.0).getSample(obsSize)
x_obs = y_obs
RWMHsampler = ot.RandomWalkMetropolisHastings(prior, initialState, instrumental)
RWMHsampler.setLikelihood(conditional, y_obs, model, x_obs)
print(
    "Log-likelihood of thetaTrue = {!r}".format(
        RWMHsampler.computeLogLikelihood(thetaTrue)
    )
)
# produces an error with current master branch
real_504 = RWMHsampler.getRealization()
print("With 504 observations, getRealization() produces {!r}".format(real_504[0]))
ott.assert_almost_equal(real_504[0], 2.0)


# this example throws in ot 1.19 as it tries to evaluate the likelihood outside the support of the prior
# see MetropolisHastingsImplementation::computeLogPosterior
obs = ot.TruncatedNormal(0.5, 0.5, 0.0, 10.0).getSample(50)
likelihood = ot.GeneralizedPareto()
prior = ot.JointDistribution([ot.LogUniform(-1.40, 4.0), ot.Normal(), ot.Normal()])
proposals = [
    ot.Uniform(
        -prior.getMarginal(k).getStandardDeviation()[0],
        +prior.getMarginal(k).getStandardDeviation()[0],
    )
    for k in range(prior.getDimension())
]
initialState = prior.getMean()
mh_coll = [
    ot.RandomWalkMetropolisHastings(prior, initialState, proposals[i], [i])
    for i in range(2)
]
for mh in mh_coll:
    mh.setLikelihood(likelihood, obs)
sampler = ot.Gibbs(mh_coll)
parameters_sample = sampler.getSample(2000)


# Trick RandomWalkMetropolisHastings into being a simple random walk
# with Uniform(-1, 1) step: every "proposal" is automatically accepted.
logdensity = ot.SymbolicFunction("x", "1")
support = ot.Interval([-inf], [inf])
proposal = ot.Uniform(-1.0, 1.0)
rw = ot.RandomWalkMetropolisHastings(logdensity, support, [0.0], proposal)

# The acceptance rate is 1 in this trivial case,
# so every adaptation step will multiply the adaptation factor
# by the expansion factor.
rw.setAdaptationExpansionFactor(2.0)
rw.setAdaptationPeriod(10)
rw.getSample(100)
ott.assert_almost_equal(rw.getAdaptationFactor(), 2.0**10, 0.0, 0.0)

# Check that the adaptation factor is really taken into account.
# We lengthen the adaptation period to get a longer period witout adaptation.
# We then compare the standard deviation of the step lengths with
# their theoretical standard deviation considering the 1024 adaptation factor.
rw.setAdaptationPeriod(100)
constantAdapationFactorSample = rw.getSample(99)
steps = constantAdapationFactorSample[1:] - constantAdapationFactorSample[:-1]
ref_std = ot.Uniform(-(2.0**10), 2.0**10).getStandardDeviation()
ott.assert_almost_equal(steps.computeStandardDeviation(), ref_std, 0.1, 0.0)

# At the next realization, once again the adaptation factor is multiplied by 2.
rw.getRealization()

# We now change the adaptation range
# to an interval with lower bound larger than 1 (the acceptance rate)
# This way, every adaptation step will multiply the adaptation factor
# by the shrink factor.
rw.setAdaptationRange(ot.Interval(1.1, 1.2))
rw.setAdaptationPeriod(10)
rw.setAdaptationShrinkFactor(0.5)
rw.getSample(100)
ott.assert_almost_equal(rw.getAdaptationFactor(), 2.0, 0.0, 0.0)



#! /usr/bin/env python

import openturns as ot
import os

ot.TESTPREAMBLE()

fname = "sample.csv"

# 1st sample
with open(fname, "w") as f:
    f.write("X1;X2;X3;X4\n-1.2;2.3;3.4;-4.5\n5.6;-6.7;7.8;8.9\n-0.1;3.2;5.1;7.5\n")
aSample = ot.Sample.ImportFromCSVFile(fname, ";")
aSample.setName("a good sample")
print("aSample=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "semi/csv"

aSample = ot.Sample.ImportFromTextFile(fname, ";")
aSample.setName("a good sample")
print("aSample=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "semi/txt"

# 2nd sample
with open(fname, "w") as f:
    f.write("X1,X2,X3,X4\n-1.2,2.3,3.4,-4.5\n5.6,-6.7,7.8,8.9\n-0.1,3.2,5.1,7.5\n")
aSample = ot.Sample.ImportFromCSVFile(fname, ",")
aSample.setName("a good comma separated sample")
print("aSample=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "comma/csv"

aSample = ot.Sample.ImportFromTextFile(fname, ",")
aSample.setName("a good comma separated sample")
print("aSample=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "comma/txt"

# export
aSample.exportToCSVFile(fname)
with open(fname) as f:
    print((f.read()))

# import with quotes in description
aSample = ot.Sample.ImportFromTextFile(fname, ";")
print("import with quotes:", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "quotes"

# spaces in description
with open(fname, "w") as f:
    f.write(
        'nos sbef saft sbeaf mid "qnos" "qsbef" "qsaft" "qsbeaf" "mid mid"\n1 2 3 4 5 6 7 8 9 10\n'
    )
aSample = ot.Sample.ImportFromTextFile(fname)
print("import with spaces:", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (1, 10), "spaces"

# spaces in description / semicolon
with open(fname, "w") as f:
    f.write(
        'nos; sbef;saft ; sbeaf ;mid mid;"qnos"; "qsbef";"qsaft"; "qsbeaf" ;"mid mid"\n1;2;3;4;5;6;7;8;9;10\n'
    )
aSample = ot.Sample.ImportFromTextFile(fname, ";")
print("import with spaces semicolon:", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (1, 10), "spaces/semicolon"

# 3rd sample
with open(fname, "w") as f:
    f.write("X1 X2 X3 X4\n-1.2 2.3 3.4 -4.5\n5.6 -6.7 7.8 8.9\n-0.1 3.2 5.1 7.5\n")

aSample = ot.Sample.ImportFromTextFile(fname)
aSample.setName("a good sample with spaces")
print("aSample=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "good/spaces"

# 4th sample
with open(fname, "w") as f:
    f.write("-1.2;2.3;3.4;-4.5\n5.6;-xxx;7.8;8.9\n-0.1;3.2;..1;7.5\n0.9;9.8;8.4;5.4\n")
aSample = ot.Sample.ImportFromCSVFile(fname, ";")
aSample.setName("a sample with bad entries")
print("aSample with bad entries (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (4, 4), "bad/csv"

aSample = ot.Sample.ImportFromTextFile(fname, ";")
aSample.setName("a sample with bad entries")
print("aSample with bad entries (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (4, 4), "bad/txt"

# 5th sample
with open(fname, "w") as f:
    f.write("-1.2;2.3;3.4;-4.5\n5.6;-6.7;7.8\n-0.1;3.2;;7.5\n6.5;9.0;7.3;-3.7\n")
aSample = ot.Sample.ImportFromCSVFile(fname, ";")
aSample.setName("a sample with missing entries")
print("aSample with missing entries (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (4, 4), "missing/csv"

aSample = ot.Sample.ImportFromTextFile(fname, ";")
aSample.setName("a sample with missing entries")
print("aSample with missing entries (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (4, 4), "missing/txt"

# 6th sample
with open(fname, "w") as f:
    f.write(
        '"X1;";"X2!()#{}%&<=>^$+-*./:\\|`?";"X3 [unit]"\n5.6;-6.7;7.8\n-0.1;3.2;7.5 \n'
    )
aSample = ot.Sample.ImportFromCSVFile(fname, ";")
aSample.setName("a sample with special chars")
print("aSample with special chars (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (2, 3), "special/csv"

aSample = ot.Sample.ImportFromTextFile(fname, ";")
aSample.setName("a sample with special chars")
print("aSample with special chars (see log)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (2, 3), "special/txt"

# text file with comments
with open(fname, "w") as f:
    f.write(
        "# hello\n\nh1 h2 h3 h4\n-1.2 2.3 3.4 -4.5 \n#spock\n5.6 -6.7 7.8 8.9 \n-0.1 3.2 5.1 7.5 "
    )
aSample = ot.Sample.ImportFromTextFile(fname)
print("aSample with comments=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "comments"

# DOS EOLs
with open(fname, "w") as f:
    f.write("-1.2;2.3;3.4;-4.5\r\n5.6;-6.7;7.8;8.9\r\n-0.1;3.2;7.5;4.6\r\n")
aSample = ot.Sample.ImportFromTextFile(fname, ";")
print("aSample from CRLF file=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "dos"

# skip lines
aSample = ot.Sample.ImportFromTextFile(fname, ";", 1)
print("import with skipped lines=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (2, 4), "skipped"

# skip lines, with desc
with open(fname, "w") as f:
    f.write("X1;X2;X3;X4\n-1.2;2.3;3.4;-4.5\n5.6;-6.7;7.8;8.9\n-0.1;3.2;7.5;4.6\n")
aSample = ot.Sample.ImportFromTextFile(fname, ";", 1)
print("import with skipped lines, desc=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "skipped/desc"

# description with an empty component
with open(fname, "w") as f:
    f.write("a,,c\n1.0,2.0,3.0")
aSample = ot.Sample.ImportFromCSVFile(fname, ",")
print("aSample description empty component=", aSample)
assert (aSample.getSize(), aSample.getDimension()) == (1, 3), "empty/csv"
assert aSample.getDescription()[1].startswith("Unnamed")
aSample = ot.Sample.ImportFromTextFile(fname, ",")
print("aSample description empty component=", aSample)
assert (aSample.getSize(), aSample.getDimension()) == (1, 3), "empty/txt"
assert aSample.getDescription()[1].startswith("Unnamed")

# check export with nan/inf
values = ["-2.4e-08", "nan", "NaN", "inf", "Inf", "-inf"]
aSample = ot.Sample([[float(v)] for v in values])
aSample.exportToCSVFile(fname)
with open(fname) as f:
    print(f.read())

# check import with nan/inf
with open(fname, "w") as f:
    for v in values:
        f.write(v + "\n")
aSample = ot.Sample.ImportFromTextFile(fname)
print("aSample with nan/inf (text)=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (6, 1), "nan/csv"
for i in range(len(values)):
    assert str(aSample[i, 0]) == str(float(values[i])), f"{aSample[i, 0]} {values[i]}"
aSample = ot.Sample.ImportFromCSVFile(fname)
print("aSample with nan/inf  (csv)=", repr(aSample).replace("(ind)", ""))
assert (aSample.getSize(), aSample.getDimension()) == (6, 1), "nan/txt"
for i in range(len(values)):
    assert str(aSample[i, 0]) == str(float(values[i])), f"{aSample[i, 0]} {values[i]}"

# "Le French way"
with open(fname, "w") as f:
    f.write("X1;X2;X3;X4\n-1,2;2,3;3,4;-4,5\n5,6;-6,7;7,8;8,9\n-0,1;3,2;5,1;7,5\n")
aSample = ot.Sample.ImportFromTextFile(fname, ";", 0, ",")
print("aSample from fr_FR.utf-8 file=", repr(aSample))
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "fr/txt"
assert aSample[0, 1] == 2.3, f"fr {aSample[0, 1]}"

# export in fr too
aSample.exportToCSVFile(fname, ";", ",")
with open(fname, "r") as f:
    print("export fr=", f.readlines()[1])

# unicode description
with open(fname, "w", encoding="utf-8") as f:
    f.write(
        "Pmax (bar);Tmax (°C)\n1.4138771410089075e+02;6.1158681591754430e+01\n3.3836092350059448e+02;1.0205155574810637e+02\n"
    )
aSample = ot.Sample.ImportFromCSVFile(fname, ";")
assert (aSample.getSize(), aSample.getDimension()) == (2, 2), "unicode desc"
assert aSample.getDescription()[1] == "Tmax (°C)"

os.remove(fname)

# unicode file name
fname = "Données"
with open(fname, "w") as f:
    f.write("X1;X2;X3;X4\n-1.2;2.3;3.4;-4.5\n5.6;-6.7;7.8;8.9\n-0.1;3.2;5.1;7.5\n")
aSample = ot.Sample.ImportFromTextFile(fname, ";")
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "unicode/text"

aSample.exportToCSVFile(fname)
aSample = ot.Sample.ImportFromCSVFile(fname)
assert (aSample.getSize(), aSample.getDimension()) == (3, 4), "unicode/csv"

# empty components + non-empty parsable values should not land in the header
with open(fname, "w") as f:
    f.write(";;8\n;;2\n;;0")
aSample = ot.Sample.ImportFromCSVFile(fname)
assert (aSample.getSize(), aSample.getDimension()) == (3, 3), "parsable"

# reject nan in header
with open(fname, "w") as f:
    f.write("1;2;nan\n1;2;3\n4;5;6")
aSample = ot.Sample.ImportFromCSVFile(fname)
assert (aSample.getSize(), aSample.getDimension()) == (3, 3), "nan header"

# all-nan is ok
with open(fname, "w") as f:
    f.write("X1;X2;X3\nnan;nan;nan\n")
aSample = ot.Sample.ImportFromCSVFile(fname)
assert (aSample.getSize(), aSample.getDimension()) == (1, 3), "all nan"

os.remove(fname)



"""
Estimate Sobol' indices for the beam by simulation algorithm
============================================================
"""

# %%
#
# In this example, we estimate the Sobol' indices for the
# :ref:`cantilever beam <use-case-cantilever-beam>` by simulation algorithm.
#

# %%
# Introduction
# ------------
#
# In this example we are going to quantify the correlation between the input
# variables and the output variable of a model thanks to Sobol indices.
#
# Sobol indices are designed to evaluate the importance of a single variable
# or a specific set of variables.
# Here the Sobol indices are estimated by sampling from the distributions of
# the input variables and propagating uncertainty through a function.
#
# In theory, Sobol indices range from 0 to 1; the closer an index value is
# to 1, the better the associated input variable explains the function output.
#
# Let us denote by :math:`d` the input dimension of the model.
#
# Sobol' indices can be computed at different orders.
#
# * First order indices evaluate the importance of one input variable
#   at a time.
#
# * Total indices give the relative importance of one input variable
#   and all its interactions with other variables.
#   Alternatively, they can be viewed as measuring how much wriggle room
#   remains to the output when all but one input variables are fixed.
#
# * In general, we are only interested in first order and total Sobol' indices.
#   There are situations, however, where we want to estimate interactions.
#   Second order indices evaluate the importance of every pair of input variables.
#   The number of second order indices is:
#
# .. math::
#    \binom{d}{2} = \frac{d \times \left( d-1\right)}{2}.
#
# In practice, when the number of input variables is not small (say,
# when :math:`d>5`), then the number of second order indices is too large
# to be easily analyzed.
# In these situations, we limit the analysis to the first order and total
# Sobol' indices.

# %%
# Define the model
# ----------------

# %%
from openturns.usecases import cantilever_beam
import openturns as ot
import openturns.viewer as otv


# %%
# We load the distribution and model from the example:
beam = cantilever_beam.CantileverBeam()
distribution = beam.independentDistribution
model = beam.model

# %%
# Estimate the Sobol' indices
# ---------------------------

# %%
# We first create the algorithm from `SobolSimulationAlgorithm` with the Saltelli estimator
# it will allow one to control the number of evaluations by convergence instead of using a fixed-size experiment
estimator = ot.SaltelliSensitivityAlgorithm()
estimator.setUseAsymptoticDistribution(True)
algo = ot.SobolSimulationAlgorithm(distribution, model, estimator)
algo.setMaximumOuterSampling(50)  # number of iterations
algo.setExperimentSize(1000)  # size of Sobol experiment at each iteration
algo.setBlockSize(4)  # number of points evaluated simultaneously
# algo.setIndexQuantileLevel(0.05)  # alpha
# algo.setIndexQuantileEpsilon(1e-2)  # epsilon
algo.run()

# %%
# Extract the results
# -------------------
result = algo.getResult()
fo = result.getFirstOrderIndicesEstimate()
to = result.getTotalOrderIndicesEstimate()
foDist = result.getFirstOrderIndicesDistribution()
graph = result.draw()
_ = otv.View(graph)

# %%
# Using a different estimator
# ---------------------------
#
# We have used the `SaltelliSensitivityAlgorithm` class to estimate the indices.
# Others are available in the library:
#
# * `SaltelliSensitivityAlgorithm`
# * `MartinezSensitivityAlgorithm`
# * `JansenSensitivityAlgorithm`
# * `MauntzKucherenkoSensitivityAlgorithm`
#

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Instantiate one distribution object
dim = 3
R = ot.CorrelationMatrix(dim)
for i in range(dim - 1):
    R[i, i + 1] = 0.25

copula = ot.NormalCopula(R)
print("Copula ", repr(copula))
print("Copula ", copula)
print("Mean ", repr(copula.getMean()))
print("Covariance ", repr(copula.getCovariance()))

# Is this copula an elliptical distribution?
print("Elliptical distribution= ", copula.isElliptical())

# Is this copula elliptical ?
print("Elliptical copula= ", copula.hasEllipticalCopula())

# Is this copula independent ?
print("Independent copula= ", copula.hasIndependentCopula())

# Test for realization of distribution
oneRealization = copula.getRealization()
print("oneRealization=", repr(oneRealization))

# Test for sampling
size = 10
oneSample = copula.getSample(size)
print("oneSample=", repr(oneSample))

# Define a point
point = ot.Point(dim, 0.2)

# Show PDF and CDF of point
pointPDF = copula.computePDF(point)
pointCDF = copula.computeCDF(point)
print("Point = ", repr(point), " pdf=%.6f" % pointPDF, " cdf=%.6f" % pointCDF)

# Get 50% quantile
quantile = copula.computeQuantile(0.5)
print("Quantile=", repr(quantile))
print("CDF(quantile)=%.6f" % copula.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(copula.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print(
    "Survival(inverseSurvival)=%.6f" % copula.computeSurvivalFunction(inverseSurvival)
)
print("entropy=%.6f" % copula.computeEntropy())
# Confidence regions
if copula.getDimension() <= 2:
    threshold = ot.Point()
    print(
        "Minimum volume interval=", copula.computeMinimumVolumeInterval(0.95, threshold)
    )
    print("threshold=", threshold)
    beta = ot.Point()
    levelSet = copula.computeMinimumVolumeLevelSet(0.95, beta)
    print("Minimum volume level set=", levelSet)
    print("beta=", beta)
    print(
        "Bilateral confidence interval=",
        copula.computeBilateralConfidenceInterval(0.95, beta),
    )
    print("beta=", beta)
    print(
        "Unilateral confidence interval (lower tail)=",
        copula.computeUnilateralConfidenceInterval(0.95, False, beta),
    )
    print("beta=", beta)
    print(
        "Unilateral confidence interval (upper tail)=",
        copula.computeUnilateralConfidenceInterval(0.95, True, beta),
    )
    print("beta=", beta)

# Covariance and correlation
covariance = copula.getCovariance()
print("covariance=", covariance)
correlation = copula.getCorrelation()
print("correlation=", correlation)
spearman = copula.getSpearmanCorrelation()
print("spearman=", spearman)
kendall = copula.getKendallTau()
print("kendall=", kendall)
x = 0.6
y = [0.2] * (dim - 1)
print("conditional PDF=%.6f" % copula.computeConditionalPDF(x, y))
print("conditional CDF=%.6f" % copula.computeConditionalCDF(x, y))
print("conditional quantile=%.6f" % copula.computeConditionalQuantile(x, y))
pt = ot.Point([0.1 * i + 0.05 for i in range(dim)])
print("sequential conditional PDF=", copula.computeSequentialConditionalPDF(point))
resCDF = copula.computeSequentialConditionalCDF(pt)
print("sequential conditional CDF(", pt, ")=", resCDF)
print(
    "sequential conditional quantile(",
    resCDF,
    ")=",
    copula.computeSequentialConditionalQuantile(resCDF),
)

# Extract the marginals
for i in range(dim):
    margin = copula.getMarginal(i)
    print("margin=", repr(margin))
    print("margin PDF=%.6f" % margin.computePDF(ot.Point(1, 0.25)))
    print("margin CDF=%.6f" % margin.computeCDF(ot.Point(1, 0.25)))
    print("margin quantile=", repr(margin.computeQuantile(0.95)))
    print("margin realization=", repr(margin.getRealization()))

# Extract a 2-D marginal
indices = [1, 0]
print("indices=", repr(indices))
margins = copula.getMarginal(indices)
print("margins=", repr(margins))
print("margins PDF=%.6f" % margins.computePDF(ot.Point(2, 0.25)))
print("margins CDF=%.6f" % margins.computeCDF(ot.Point(2, 0.25)))
quantile = ot.Point(margins.computeQuantile(0.95))
print("margins quantile=", repr(quantile))
print("margins CDF(qantile)=%.6f" % margins.computeCDF(quantile))
print("margins realization=", repr(margins.getRealization()))

# Creation of the correlation matrix from a Spearman correlation matrix
spearman = ot.CorrelationMatrix(dim)
for i in range(1, dim):
    spearman[i, i - 1] = 0.25

correlation = ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(spearman)
print(
    "Normal copula correlation=",
    repr(correlation),
    " from the Spearman correlation=",
    repr(spearman),
)

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(copula)
validation.run()

# computeProbability
spearman_corr = ot.CorrelationMatrix([[1.0, 0.74], [0.74, 1.0]])
copula = ot.NormalCopula(
    ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(spearman_corr)
)
interval = ot.Interval([0.958722, 0.902063], [1.0, 1.0])
prob = copula.computeProbability(interval)
print("prob=%.6f" % prob)

print(ot.NormalCopula(1).getParametersCollection())

# normal quantile inf propagation bug in iso transfo
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.2
copula = ot.NormalCopula(R)
standard_list = [ot.Uniform(), ot.Beta()]
distribution = ot.JointDistribution(standard_list, copula)
standard = ot.JointDistribution(standard_list)
transformation = ot.DistributionTransformation(distribution, standard)
standard_sample = transformation(ot.Normal(distribution.getDimension()).getSample(10))
print(standard_sample)



"""
Kolmogorov-Smirnov : get the statistics distribution
====================================================
"""

# %%

# %%
# In this example, we draw the Kolmogorov-Smirnov (KS) distribution for a sample size 10.
# We want to test the hypothesis that this sample has the `Uniform(0, 1)`
# distribution.
# The K.S. distribution is first plotted in the case where the
# parameters of the uniform distribution are known.
# Then we plot the distribution when the parameters of the uniform
# distribution are estimated from the sample.
#
# *Reference* : Hovhannes Keutelian, "The Kolmogorov-Smirnov test when
# parameters are estimated from data", 30 April 1991, Fermilab
#
# Note: There is a sign error in the paper; the equation:
# `D[i]=max(abs(S+step),D[i])` must be replaced with `D[i]=max(abs(S-step),D[i])`.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
x = [0.9374, 0.7629, 0.4771, 0.5111, 0.8701, 0.0684, 0.7375, 0.5615, 0.2835, 0.2508]
sample = ot.Sample([[xi] for xi in x])

# %%
samplesize = sample.getSize()
samplesize

# %%
# Plot the empirical distribution function.

# %%
graph = ot.FiniteDiscreteDistribution(sample).drawCDF()
graph.setLegends(["Sample"])
curve = ot.Curve([0, 1], [0, 1])
curve.setLegend("Uniform")
graph.add(curve)
graph.setXTitle("X")
graph.setTitle("Cumulative distribution function")
view = otv.View(graph)


# %%
# The `computeKSStatisticsIndex` function computes the Kolmogorov-Smirnov
# distance between the sample and the distribution.
# The following function is for teaching purposes only: use `FittingTest`
# for real applications.


# %%
def computeKSStatistics(sample, distribution):
    sample = sample.sort()
    n = sample.getSize()
    D = 0.0
    D_previous = 0.0
    for i in range(n):
        F = distribution.computeCDF(sample[i])
        Fminus = F - float(i) / n
        Fplus = float(i + 1) / n - F
        D = max(Fminus, Fplus, D)
        if D > D_previous:
            D_previous = D
    return D


# %%
dist = ot.Uniform(0, 1)
dist

# %%
computeKSStatistics(sample, dist)


# %%
# The following function generates a sample of K.S. distances when the tested distribution is the `Uniform(0,1)` distribution.


# %%
def generateKSSampleKnownParameters(nrepeat, samplesize):
    """
    nrepeat : Number of repetitions, size of the table
    samplesize : the size of each sample to generate from the Uniform distribution
    """
    dist = ot.Uniform(0, 1)
    D = ot.Sample(nrepeat, 1)
    for i in range(nrepeat):
        sample = dist.getSample(samplesize)
        D[i, 0] = computeKSStatistics(sample, dist)
    return D


# %%
# Generate a sample of KS distances.

# %%
nrepeat = 10000  # Size of the KS distances sample
sampleD = generateKSSampleKnownParameters(nrepeat, samplesize)


# %%
# Compute exact Kolmogorov CDF.


# %%
def pKolmogorovPy(x):
    y = ot.DistFunc.pKolmogorov(samplesize, x[0])
    return [y]


# %%
pKolmogorov = ot.PythonFunction(1, 1, pKolmogorovPy)


# %%
def dKolmogorov(x, samplesize):
    """
    Compute Kolmogorov PDF for given x.
    x : an array, the points where the PDF must be evaluated
    samplesize : the size of the sample
    Reference
    Numerical Derivatives in Scilab, Michael Baudin, May 2009
    """
    n = x.getSize()
    y = ot.Sample(n, 1)
    for i in range(n):
        y[i, 0] = pKolmogorov.gradient(x[i])[0, 0]
    return y


# %%
def linearSample(xmin, xmax, npoints):
    """Returns a sample created from a regular grid
    from xmin to xmax with npoints points."""
    step = (xmax - xmin) / (npoints - 1)
    rg = ot.RegularGrid(xmin, step, npoints)
    vertices = rg.getVertices()
    return vertices


# %%
n = 1000  # Number of points in the plot
s = linearSample(0.001, 0.999, n)
y = dKolmogorov(s, samplesize)

# %%
curve = ot.Curve(s, y)
curve.setLegend("Exact distribution")
graph = ot.HistogramFactory().build(sampleD).drawPDF()
graph.setLegends(["Empirical distribution"])
graph.add(curve)
graph.setTitle("Kolmogorov-Smirnov distribution (known parameters)")
graph.setXTitle("KS-Statistics")
view = otv.View(graph)


# %%
# Known parameters versus estimated parameters
# --------------------------------------------

# %%
# The following function generates a sample of K.S. distances when the tested
# distribution is the `Uniform(a,b)` distribution, where the `a` and `b`
# parameters are estimated from the sample.


# %%
def generateKSSampleEstimatedParameters(nrepeat, samplesize):
    """
    nrepeat : Number of repetitions, size of the table
    samplesize : the size of each sample to generate from the Uniform distribution
    """
    distfactory = ot.UniformFactory()
    refdist = ot.Uniform(0, 1)
    D = ot.Sample(nrepeat, 1)
    for i in range(nrepeat):
        sample = refdist.getSample(samplesize)
        trialdist = distfactory.build(sample)
        D[i, 0] = computeKSStatistics(sample, trialdist)
    return D


# %%
# Generate a sample of KS distances.

# %%
sampleDP = generateKSSampleEstimatedParameters(nrepeat, samplesize)

# %%
graph = ot.KernelSmoothing().build(sampleD).drawPDF()
graph.setLegends(["Known parameters"])
graphP = ot.KernelSmoothing().build(sampleDP).drawPDF()
graphP.setLegends(["Estimated parameters"])
graph.add(graphP)
graph.setTitle("Kolmogorov-Smirnov distribution")
graph.setXTitle("KS-Statistics")
view = otv.View(graph)

# %%
# Display the graphs
view.ShowAll()


# %%
# We see that the distribution of the KS distances when the parameters are
# estimated is shifted towards the left: smaller distances occur more often.
# This is a consequence of the fact that the estimated parameters tend to make
# the estimated distribution closer to the empirical sample.



#! /usr/bin/env python
"""
Consider a model exactly linear with respect to the parameters.
In this case, the LLSQ calibration performs as good as it can.
"""

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

ot.RandomGenerator.SetSeed(0)


def modelLineaire(X):
    x, theta1, theta2, theta3 = X
    y = theta1 + theta2 * x + theta3 * x**2
    return [y]


g = ot.PythonFunction(4, 1, modelLineaire)

trueParameter = ot.Point([12.0, 7.0, -8])

parameterDimension = trueParameter.getDimension()

X = ot.Uniform()
Theta1 = ot.Dirac(trueParameter[0])
Theta2 = ot.Dirac(trueParameter[1])
Theta3 = ot.Dirac(trueParameter[2])

inputRandomVector = ot.JointDistribution([X, Theta1, Theta2, Theta3])

candidate = ot.Point([8.0, 9.0, -6.0])

calibratedIndices = [1, 2, 3]
model = ot.ParametricFunction(g, calibratedIndices, candidate)

outputObservationNoiseSigma = 2.0  # (Pa)
observationOutputNoise = ot.Normal(0.0, outputObservationNoiseSigma)

size = 1000

# Generate exact outputs
inputSample = inputRandomVector.getSample(size)
outputStress = g(inputSample)
# Add noise
sampleNoiseH = observationOutputNoise.getSample(size)
outputObservations = outputStress + sampleNoiseH
# Calibrate
inputObservations = inputSample[:, 0]
algo = ot.LinearLeastSquaresCalibration(
    model, inputObservations, outputObservations, candidate, "SVD"
)
algo.run()
calibrationResult = algo.getResult()

# Check residual distribution
residualDistribution = calibrationResult.getObservationsError()
meanResidual = residualDistribution.getMean()[0]
assert meanResidual == 0.0
sigmaResidual = residualDistribution.getStandardDeviation()[0]
rtol = 0.0
atol = 5.0e-2
ott.assert_almost_equal(sigmaResidual, outputObservationNoiseSigma, rtol, atol)

# Check other fields
print("result=", calibrationResult)



"""
Create a random walk process
============================
"""

# %%
# This example details first how to create and manipulate a random walk.
#
# A random walk :math:`X: \Omega \times \mathcal{D} \rightarrow \mathbb{R}^d` is a process
# where :math:`\mathcal{D}=\mathbb{R}` discretized on the time grid :math:`(t_i)_{i \geq 0}` such
# that:
#
# .. math::
#    \begin{aligned}
#      X_{t_0} & = & \vect{x}_{t_0} \\
#      \forall n>0,\: X_{t_n} & = & X_{t_{n-1}} + \varepsilon_{t_n}
#    \end{aligned}
#
# where :math:`\vect{x}_0 \in \mathbb{R}^d` and :math:`\varepsilon` is a white noise of
# dimension :math:`d`.
#
# The library proposes to model it through the object :class:`~openturns.RandomWalk` defined
# thanks to the origin, the distribution of the white noise and the time
# grid.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Define the origin
origin = [0.0]

# %%
# Define an 1-d mesh
tgrid = ot.RegularGrid(0.0, 1.0, 500)

# %%
# 1-d random walk and discrete distribution
dist = ot.FiniteDiscreteDistribution([[-1], [10]], [0.9, 0.1])
process = ot.RandomWalk(origin, dist, tgrid)
sample = process.getSample(5)
graph = sample.drawMarginal(0)
graph.setTitle("1D Random Walk with discrete steps")
view = otv.View(graph)

# %%
# 1-d random walk and continuous distribution
dist = ot.Normal(0.0, 1.0)
process = ot.RandomWalk(origin, dist, tgrid)
sample = process.getSample(5)
graph = sample.drawMarginal(0)
graph.setTitle("1D Random Walk with continuous steps")
view = otv.View(graph)

# %%
# Define the origin
origin = [0.0] * 2

# %%
# Color palette
pal = ["red", "cyan", "blue", "yellow", "green"]

# %%
# 2-d random walk and discrete distribution
dist = ot.FiniteDiscreteDistribution([[-1.0, -2.0], [1.0, 3.0]], [0.5, 0.5])
process = ot.RandomWalk(origin, dist, tgrid)
sample = process.getSample(5)
graph = ot.Graph("2D Random Walk with discrete steps", "X1", "X2")
for i in range(5):
    graph.add(ot.Curve(sample[i], pal[i % len(pal)], "solid"))
view = otv.View(graph)

# %%
# 2-d random walk and continuous distribution
dist = ot.Normal(2)
process = ot.RandomWalk(origin, dist, tgrid)
sample = process.getSample(5)
graph = ot.Graph("2D Random Walk with continuous steps", "X1", "X2")
for i in range(5):
    graph.add(ot.Curve(sample[i], pal[i % len(pal)], "solid"))
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from openturns.viewer import View
import time

ot.RandomGenerator.SetSeed(0)
ot.Log.Show(ot.Log.INFO)

# Bounds are [0,1]^dimension
dimension = 2
bounds = ot.Interval(dimension)

# Size of sample
size = 10

# Factory: lhs generates
lhsDesign = ot.LHSExperiment(
    ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension), size
)
lhsDesign.setAlwaysShuffle(True)  # randomized

geomProfile = ot.GeometricProfile(10.0, 0.999, 50000)
c2 = ot.SpaceFillingC2()
sa = ot.SimulatedAnnealingLHS(lhsDesign, geomProfile, c2)
tic = time.time()
result = sa.generate()
toc = time.time()
dt1 = toc - tic
print("time=%f" % dt1)
print("dimension=%d, size=%d,sa=%s" % (dimension, size, sa))
print(
    str(result.getOptimalValue())
    + " c2="
    + str(result.getC2())
    + " phiP="
    + str(result.getPhiP())
    + " minDist="
    + str(result.getMinDist())
)

crit = result.drawHistoryCriterion()
proba = result.drawHistoryProbability()
temp = result.drawHistoryTemperature()

pp = PdfPages("small_OTLHS.pdf")
# Criterion
fig = View(crit, plot_kwargs={"color": "blue"}).getFigure()
fig.savefig("crit_sa_geom.png")
pp.savefig(fig)
plt.close(fig)
# Proba
fig = View(
    proba, plot_kwargs={"marker": "o", "ms": 0.6}, axes_kwargs={"ylim": [-0.05, 1.05]}
).getFigure()
fig.savefig("lhs_c2_proba.png")
pp.savefig(fig)
plt.close(fig)
# Temperature
fig = View(temp).getFigure()
pp.savefig(fig)
plt.close(fig)

linearProfile = ot.LinearProfile(10.0, 50000)
minDist = ot.SpaceFillingMinDist()
sa = ot.SimulatedAnnealingLHS(lhsDesign, linearProfile, minDist)
tic = time.time()
result = sa.generate()
toc = time.time()
dt2 = toc - tic
print("time=%f" % dt2)
print("dimension=%d, size=%d,sa=%s" % (dimension, size, sa))
print(
    str(result.getOptimalValue())
    + " c2="
    + str(result.getC2())
    + " phiP="
    + str(result.getPhiP())
    + " minDist="
    + str(result.getMinDist())
)

crit = result.drawHistoryCriterion()
proba = result.drawHistoryProbability()
temp = result.drawHistoryTemperature()

# Criterion
fig = View(crit, plot_kwargs={"color": "blue"}).getFigure()
fig.savefig("crit_sa_lin.png")
pp.savefig(fig)
plt.close(fig)
# Proba
fig = View(
    proba, plot_kwargs={"marker": "o", "ms": 0.6}, axes_kwargs={"ylim": [-0.05, 1.05]}
).getFigure()
fig.savefig("lhs_mindist_proba.png")
pp.savefig(fig)
plt.close(fig)
# Temperature
fig = View(temp).getFigure()
pp.savefig(fig)
plt.close(fig)

pp.close()



#! /usr/bin/env python

import openturns as ot


# DEFAULT CONSTRUCTOR AND STRING CONVERTER
print("test : default constructor and string converter")

# Default constructor
squareComplexMatrix0 = ot.SquareComplexMatrix()

# String converter
print("squareComplexMatrix0 = ", repr(squareComplexMatrix0))

# CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
print("test : constructor with size, operator() and string converter")

# Constructor with size
squareComplexMatrix1 = ot.SquareComplexMatrix(2)

# Check operator() methods
squareComplexMatrix1[0, 0] = 1.0 + 1j
squareComplexMatrix1[0, 1] = 3.0 + 1j
squareComplexMatrix1[1, 0] = 1.0j
squareComplexMatrix1[1, 1] = 5.0 + 1.0j

# String converter
print("squareComplexMatrix1 = ", repr(squareComplexMatrix1))

# COPY CONSTRUCTOR AND STRING CONVERTER
print("test : copy constructor and string converter")

# Copy constructor
squareComplexMatrix2 = ot.SquareComplexMatrix(squareComplexMatrix1)

# String converter
print("squareComplexMatrix2 = ", repr(squareComplexMatrix2))

# GET DIMENSIONS METHODS
print("test : get dimensions methods")

# Get dimension methods
print("squareComplexMatrix1's nbRows = ", squareComplexMatrix1.getNbRows())
print("squareComplexMatrix1's nbColumns = ", squareComplexMatrix1.getNbColumns())

# CONJUGATE METHOD
print("test  : conjugate method")

# Check conjugate method
squareComplexMatrix4 = squareComplexMatrix1.conjugate()
print("squareComplexMatrix1 conjugate = ", repr(squareComplexMatrix4))

#  ADDITION METHOD
print("test : addition method")

# Check addition method : we check the operator and the symmetry of the
# operator, thus testing the comparison operator
sum1 = squareComplexMatrix1 + squareComplexMatrix4
sum2 = squareComplexMatrix4 + squareComplexMatrix1
print("sum1 = ", repr(sum1))
print("sum2 = ", repr(sum2))
print("sum1 equals sum2 = ", sum1 == sum2)

# SUBTRACTION METHOD
print("test : subtraction method")

# Check subtraction method
diff = squareComplexMatrix1 - squareComplexMatrix4
print("diff = ", repr(diff))

#  MATRIX MULTIPLICATION METHOD
print("test : matrix multiplication method")

# Check multiplication method
prod = squareComplexMatrix1 * squareComplexMatrix4
print("prod = ", repr(prod))

# MULTIPLICATION WITH A NUMERICAL POINT METHOD
print("test : multiplication with a numerical point method")

# Create the numerical point
pt = ot.ComplexCollection(2)
pt[0] = 1.0 + 1j
pt[1] = 1j
print("pt = ", repr(pt))

# Check the product method
ptResult = squareComplexMatrix1 * pt
print("ptResult = ", repr(ptResult))

# MULTIPLICATION AND DIVISION BY A NUMERICAL SCALAR METHODS
print("test : multiplication and division by a numerical scalar methods")

# Check the multiplication method
s = 3.0 + 2j
scalprod = squareComplexMatrix1 * s

print("scalprod = ", repr(scalprod))

# Check the division method
scaldiv1 = squareComplexMatrix1 / s
scaldiv2 = squareComplexMatrix1 / s
print("scaldiv1 = ", repr(scaldiv1))
print("scaldiv2 = ", repr(scaldiv2))
print("scaldiv1 equals scaldiv2 = ", (scaldiv1 == scaldiv2))

# ISEMPTY METHOD
print("test : isEmpty method")

# Check method isEmpty
squareComplexMatrix5 = ot.SquareComplexMatrix()
squareComplexMatrix6 = ot.SquareComplexMatrix()
print("squareComplexMatrix0 is empty = ", squareComplexMatrix0.isEmpty())
print("squareComplexMatrix1 is empty = ", squareComplexMatrix1.isEmpty())
print("squareComplexMatrix5 is empty = ", squareComplexMatrix5.isEmpty())



import openturns as ot
import openturns.viewer as otv

f = ot.SymbolicFunction(["x"], ["x * sin(x)"])
sampleX = [[1.0], [2.0], [3.0], [4.0], [5.0], [6.0], [7.0], [8.0]]
sampleY = f(sampleX)
basis = ot.Basis([ot.SymbolicFunction(["x"], ["1"])])
covarianceModel = ot.SquaredExponential([1.0])
covarianceModel.setActiveParameter([])
fitter_algo = ot.GaussianProcessFitter(sampleX, sampleY, covarianceModel, basis)
fitter_algo.run()
fitter_result = fitter_algo.getResult()

graph = f.draw(0.0, 10.0)
graph.add(fitter_result.getMetaModel().draw(0.0, 10.0))
graph.add(ot.Cloud(sampleX, sampleY))

algo = ot.GaussianProcessRegression(fitter_result)
algo.run()
result = algo.getResult()
fhat = result.getMetaModel()

graph.add(fhat.draw(0.0, 10.0))
graph.setLegends(["model", "GP Fitter", "sample", "GP Regression"])
graph.setLegendPosition("upper left")
graph.setTitle(r"$y(x)=x * sin(x)$")
graph.setYTitle(r"$f$")
graph.setXTitle(r"$x$")
otv.View(graph, figure_kw={"figsize": (8, 4)})



"""
Calibration of the logistic model
=================================

We present a calibration study of the logistic growth model defined :ref:`here <use-case-logistic>`.
The data set that we use is the U.S. population from 1790 to 2000.
One of the specific properties of this study is that the observations
that we use are real world observations.
Hence when we calibrate the model on the data, there is a discrepancy that will
be seen.
In this example, we calibrate the parameters of a model which predicts
the dynamics of the size of a population.
This shows how we can calibrate a model which predicts a time
dependent output.
The output of the model is a time series representing the evolution of the
population.
This requires a transpose of the sample, so that we can benefit from
the visualization methods.

Variables
---------

In the particular situation where we want to calibrate this model, the
following list presents which variables are observed input variables,
input calibrated variables and observed output variables.

- :math:`t`: Input. Observed.
- :math:`z_0`, :math:`a`, :math:`c`: Inputs. Calibrated.
- :math:`z`: Output. Observed.

"""

# %%
# Analysis of the data
# --------------------

# %%
from openturns.usecases import logistic_model
import openturns as ot
import numpy as np
import openturns.viewer as otv
from matplotlib import pyplot as plt

# %%
# We load the logistic model from the usecases module :
lm = logistic_model.LogisticModel()
print("Inputs:", lm.model.getInputDescription())
print("Outputs: ", lm.model.getOutputDescription())

# %%
# We see that there are 24 inputs.
# The first 22 inputs are the timestamps and the last two inputs are the
# a and c parameters to calibrate.
# The 22 outputs are the population of the U.S. in millions.

# %%
# The data is based on 22 dates from 1790 to 2000. The observation points are
# stored in the `data` field :
observedSample = lm.data
observedSample[:5]

# %%
nbobs = observedSample.getSize()
nbobs

# %%
timeObservations = observedSample[:, 0]
timeObservations[0:5]

# %%
populationObservations = observedSample[:, 1]
populationObservations[0:5]

# %%
graph = ot.Graph("", "Time (years)", "Population (Millions)")
graph.setLegendPosition("upper left")
cloud = ot.Cloud(timeObservations, populationObservations)
cloud.setLegend("Observations")
cloud.setPointStyle(
    ot.ResourceMap.GetAsString("CalibrationResult-ObservationPointStyle")
)
graph.add(cloud)
curve = ot.Curve(timeObservations, populationObservations)
curve.setColor(graph.getDrawable(0).getColor())
curve.setLegend("")
curve.setLineStyle(ot.ResourceMap.GetAsString("CalibrationResult-ObservationLineStyle"))
graph.add(curve)
view = otv.View(graph)

# %%
# We see that there is a very smooth growth of the U.S. population.
# This is a good candidate for model calibration.

# %%
# We consider the times and populations as dimension 22 vectors.
# The `logisticModel` function takes a dimension 24 vector as input and
# returns a dimension 22 vector.
# The first 22 components of the input vector contains the dates and the
# remaining 2 components are :math:`a` and :math:`b`.

# %%
# Print the number of dates:
print(lm.data.getSize())

# %%
# Get the physical model to calibrate.
logisticModelPy = lm.model

# %%
# The reference values of the parameters.
#
# Because :math:`b` is so small with respect to :math:`a`, we use the logarithm.
# In other words, we calibrate :math:`c = \log(b)` instead of calibrating
# :math:`b`.
# This makes the calibration much easier.

# %%
np.log(1.5587e-10)

# %%
a = 0.03134
c = -22.58
thetaPrior = [a, c]

# %%
# In the physical model, the inputs and parameters are ordered as
# presented in the next table.
# Notice that there are no parameters in the physical model.
#
# +-------+----------------+
# | Index | Input variable |
# +=======+================+
# | 0     | t1             |
# +-------+----------------+
# | 1     | t2             |
# +-------+----------------+
# | ...   | ...            |
# +-------+----------------+
# | 21    | t22            |
# +-------+----------------+
# | 22    | a              |
# +-------+----------------+
# | 23    | c              |
# +-------+----------------+
#
# +-------+-----------+
# | Index | Parameter |
# +=======+===========+
# | ∅     | ∅         |
# +-------+-----------+
#
# **Table 1.** Indices and names of the inputs and parameters of the physical model.
#
print("Physical Model Inputs:", lm.model.getInputDescription())
print("Physical Model Parameters:", lm.model.getParameterDescription())

# %%
# In order to perform calibration, we have to define a parametric model,
# with observed inputs and parameters to calibrate.
# In order to do this, we create a :class:`~openturns.ParametricFunction` where the parameters
# are `a` and `c` which have the indices 22 and 23 in the physical model.
#
# +-------+----------------+
# | Index | Input variable |
# +=======+================+
# | 0     | t1             |
# +-------+----------------+
# | 1     | t2             |
# +-------+----------------+
# | ...   | ...            |
# +-------+----------------+
# | 21    | t22            |
# +-------+----------------+
#
# +-------+-----------+
# | Index | Parameter |
# +=======+===========+
# | 0     | a         |
# +-------+-----------+
# | 1     | c         |
# +-------+-----------+
#
# **Table 2.** Indices and names of the inputs and parameters of the parametric model.
#

# %%
logisticParametric = ot.ParametricFunction(logisticModelPy, [22, 23], thetaPrior)
print("Parametric Model Inputs:", logisticParametric.getInputDescription())
print("Parametric Model Parameters:", logisticParametric.getParameterDescription())

# %%
# Check that we can evaluate the parametric function.

# %%
populationPredicted = logisticParametric(timeObservations.asPoint())
print(populationPredicted[:5])

# %%
graph = ot.Graph(
    "Before calibration", "Time (years)", "Population (Millions)", True, "upper left"
)
# Observations
cloud = ot.Cloud(timeObservations, populationObservations)
cloud.setLegend("Observations")
cloud.setPointStyle(
    ot.ResourceMap.GetAsString("CalibrationResult-ObservationPointStyle")
)
graph.add(cloud)
# Predictions
cloud = ot.Cloud(timeObservations.asPoint(), populationPredicted)
cloud.setLegend("Predictions")
cloud.setPointStyle(ot.ResourceMap.GetAsString("CalibrationResult-PriorPointStyle"))
graph.add(cloud)
view = otv.View(graph)

# %%
# We see that the fit is not good: the observations continue to grow after 1950, while the growth of the prediction seems to fade.

# %%
# Calibration with linear least squares
# -------------------------------------

# %%
# The calibration algorithm takes input and output samples as arguments.
# In this case, we choose to consider one single observation in dimension 22.
# In order to perform calibration, we create a `Sample` of input times which has
# one observation in dimension 22.

# %%
timeObservationsSample = ot.Sample([timeObservations.asPoint()])
timeObservationsSample[0, 0:5]

# %%
# Similarly, we create a `Sample` of output populations which has
# one observation in dimension 22.

# %%
populationObservationsSample = ot.Sample([populationObservations.asPoint()])
populationObservationsSample[0, 0:5]

# %%
logisticParametric = ot.ParametricFunction(logisticModelPy, [22, 23], thetaPrior)

# %%
# Check that we can evaluate the parametric function.

# %%
populationPredicted = logisticParametric(timeObservationsSample)
populationPredicted[0, 0:5]

# %%
#  Calibration
# ------------

# %%
# We are now able to perform the calibration, using linear least squares
# using the :class:`~openturns.LinearLeastSquaresCalibration` class.

# %%
algo = ot.LinearLeastSquaresCalibration(
    logisticParametric, timeObservationsSample, populationObservationsSample, thetaPrior
)
algo.run()
calibrationResult = algo.getResult()
thetaMAP = calibrationResult.getParameterMAP()
print("theta After = ", thetaMAP)
print("theta Before = ", thetaPrior)

# %%
# Compared to the value of the parameters before calibration, we can see that the
# parameters were significantly updated.

# %%
# In order to see if the optimum parameters are sensitive to the
# observation errors, we compute 95% confidence intervals.

# %%
thetaPosterior = calibrationResult.getParameterPosterior()
print(thetaPosterior.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)[0])

# %%
# The 95% confidence intervals of the optimum are relatively narrow:
# this means that the optimum is not very sensitive to the observation errors.
# In this case, the parameters are accurately estimated.

# %%
# In this example, we have considered a single observation in dimension 22.
# In this case, the parametric model has has 22 outputs, which
# create a large number of plots.
# Transpose samples to interpret the data as a Sample in dimension 1
# with 22 observations.

# %%
if calibrationResult.getInputObservations().getSize() == 1:
    calibrationResult.setInputObservations(timeObservations)
    calibrationResult.setOutputObservations(populationObservations)
    outputPriorMeanDimension22 = calibrationResult.getOutputAtPriorMean()
    outputAtPriorDimension1 = ot.Sample.BuildFromPoint(outputPriorMeanDimension22[0])
    outputPosteriorMeanDimension22 = calibrationResult.getOutputAtPosteriorMean()
    outputPosteriorMeanDimension1 = ot.Sample.BuildFromPoint(
        outputPosteriorMeanDimension22[0]
    )
    calibrationResult.setOutputAtPriorAndPosteriorMean(
        outputAtPriorDimension1, outputPosteriorMeanDimension1
    )

# %%
# Increase the default number of points in the plots.
# This produces smoother spiky distributions.

# %%
ot.ResourceMap.SetAsUnsignedInteger("Distribution-DefaultPointNumber", 300)

# %%
# The next plot presents the U.S. population depending on the time.
# We see that the calibrated model fits to the data more
# than the uncalibrated model, especially on the second part
# of the time interval.

# sphinx_gallery_thumbnail_number = 3
graph = calibrationResult.drawObservationsVsInputs()
graph.setLegendPosition("upper left")
view = otv.View(graph)

# %%
# As a metamodel, we can compare the predicted U.S. population
# depending on the observed U.S. population.
# We see that the model fits to the data accurately for
# a population up to 150 million people, but diverges when the population
# gets larger.
# On the other hand, the calibrated model under-predicts the
# population for the [0,150] population interval, and
# over-predicts for the [150,300] interval, balancing the errors
# so that the model globally fits.
# The "S" shape of the graph after calibration reveals that
# the calibrated model has a structure with residuals that do
# not follow a normal distribution (otherwise the calibrated
# cloud would be spread over and under the diagonal).
# In other words, the model and the data do not fit very well.

# %%
graph = calibrationResult.drawObservationsVsPredictions()
view = otv.View(graph)

# %%
# The residuals analysis shows that some residuals were very
# large before calibration.
# After calibration, most residuals are in the [-20,20] interval,
# which explains why the calibrated model fits the data better.
# We notice that the distribution of the
# calibrated residuals is relatively close to a normal distribution.
# This may show that the least squares model is appropriate in
# this case with respect to this criterion.

# %%
graph = calibrationResult.drawResiduals()
view = otv.View(graph)

# %%
# The next plot shows that there is a significant improvement
# after the calibration: the initial point is very different
# from the distribution of the optimum parameter.

# %%
graph = calibrationResult.drawParameterDistributions()
view = otv.View(
    graph,
    figure_kw={"figsize": (8.0, 4.0)},
    legend_kw={"bbox_to_anchor": (1.0, 1.0), "loc": "upper left"},
)
plt.subplots_adjust(right=0.8)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Time grid creation
Tmin = 0.0
deltaT = 0.1
steps = 11

# Initialization of the time grid timeGrid1
timeGrid = ot.RegularGrid(Tmin, deltaT, steps)

# Default aggregated process creation
myProcess = ot.AggregatedProcess()

# We fix the time grid to the process
myProcess.setTimeGrid(timeGrid)

print("myProcess=", myProcess)
print("a realization=", myProcess.getRealization())
marginalProcess = myProcess.getMarginal(0)
print("a marginal process=", marginalProcess)

# Parameter aggregated process creation
coll = [ot.WhiteNoise(), ot.ARMA(), ot.GaussianProcess()]
myProcess = ot.AggregatedProcess(coll)
myProcess.setTimeGrid(timeGrid)
print("myProcess=", myProcess)
print("a realization=", myProcess.getRealization())
marginalProcess = myProcess.getMarginal(1)
print("a marginal process=", marginalProcess)
print("another marginal process=", myProcess.getMarginal([0, 2]))



"""
Test independence
=================
"""

# %%
import openturns as ot

# %%
# Sample independence test
# ------------------------
#
# In this paragraph we perform tests to assess whether two 1-d samples generated
# by two random variables :math:`X` and :math:`Y` are independent or not.
#
# The following tests are available:
#
# - the ChiSquared test only used for discrete variables. Refer to :ref:`chi2_independence_test` for
#   more details.
#
# - the Pearson test: this test checks if there exists a linear
#   relationship between :math:`X` and :math:`Y`. It is equivalent to an independence test only
#   if the random vector :math:`(X,Y)` is a Gaussian vector. Refer to :ref:`pearson_test` for
#   more details.
#
# - the Spearman test: this test checks if there exists a monotonic
#   relationship between :math:`X` and :math:`Y`. Refer to :ref:`spearman_test` for
#   more details.
#
# - independence test using regression: this test checks if there exists a linear relation between
#   :math:`X` and :math:`Y` using a linear model.

# %%
# Case 1: Pearson and Spearman tests
# ----------------------------------
#
# We create a sample generated by a bivariate Gaussian vector :math:`(X,Y)` with independent components.
sample_Biv = ot.Normal(2).getSample(1000)
sample1 = sample_Biv.getMarginal(0)
sample2 = sample_Biv.getMarginal(1)

# %%
# To test the independence between both samples, we first use the Pearson test with
# the Type I error equal to 0.1 (which is the probability to wrongly rejects the null hypothesis).
# The Pearson test checks if there is a linear correlation between both random variables.
# The null hypothesis is: *There is no linear relation*.
# As :math:`(X,Y)` is a  Gaussian vector, it is equivalent to test the independence of the components.
resultPearson = ot.HypothesisTest.Pearson(sample1, sample2, 0.10)

# %%
# We can then display the result of the test as a yes/no answer with
# the `getBinaryQualityMeasure`. We can retrieve the p-value and the threshold with the `getPValue`
# and `getThreshold` methods.
print(
    "Is the Pearson correlation coefficient is null ?",
    resultPearson.getBinaryQualityMeasure(),
    "p-value=%.6g" % resultPearson.getPValue(),
    "threshold=%.6g" % resultPearson.getThreshold(),
)

# %%
# **Conclusion**: The Pearson test validates that there is no linear correlation between both samples:
# the null hypothesis assuming that the Pearson correlation coefficient is null is accepted. It
# means that the components are independent.
# In the general case, the Gaussian vector hypothesis must be validated!

# %%
# We can also use the Spearman test  with
# the Type I error equal to 0.1 (which is the probability to wrongly rejects the null hypothesis).
# The Spearman test checks if there exists a monotonic relationship between :math:`X` and :math:`Y`.
# The null hypothesis is: *There is no monotonic relation*.
resultSpearman = ot.HypothesisTest.Spearman(sample1, sample2, 0.10)
print(
    "Is the Spearman correlation coefficient is null ?",
    resultSpearman.getBinaryQualityMeasure(),
    "p-value=%.6g" % resultSpearman.getPValue(),
    "threshold=%.6g" % resultSpearman.getThreshold(),
)

# %%
# **Conclusion**: The Spearman test validates that there is no monotonic correlation between both samples:
# the null hypothesis assuming that the Spearman correlation coefficient is null is accepted.

# %%
# Here, we create a bivariate sample from a Gaussian vector which components are correlated. We note
# that the Pearson test and the Spearman test both detect a correlation as both null hypotheses are
# rejected.
cor_Matrix = ot.CorrelationMatrix(2)
cor_Matrix[0, 1] = 0.8
sample_Biv = ot.Normal([0] * 2, [1] * 2, cor_Matrix).getSample(1000)
sample1 = sample_Biv.getMarginal(0)
sample2 = sample_Biv.getMarginal(1)
resultPearson = ot.HypothesisTest.Pearson(sample1, sample2, 0.10)
resultSpearman = ot.HypothesisTest.Spearman(sample1, sample2, 0.10)
print("Pearson test : ", resultPearson)
print("Spearman test : ", resultSpearman)

# %%
# We consider now a discrete distribution. Let us create two independent samples.
sample1 = ot.Poisson(0.2).getSample(100)
sample2 = ot.Poisson(0.2).getSample(100)

# %%
# We use the Chi2 test to check independence.
resultChi2 = ot.HypothesisTest.ChiSquared(sample1, sample2, 0.10)

# %%
# We display the results.
print(
    "Are the components independent?",
    resultChi2.getBinaryQualityMeasure(),
    "p-value=%.6g" % resultChi2.getPValue(),
    "threshold=%.6g" % resultChi2.getThreshold(),
)

# %%
# **Conclusion**: The Chi2  test validates that both samples are independent:
# the null hypothesis assuming the independence is accepted.


# %%
# Case 2: Independence test using regression
# ------------------------------------------
#
# This test consists in fitting a linear model between :math:`X` and :math:`Y` and anylysing
# if the coefficients are significantly different from 0.

# %%
# We create a sample generated by a Gaussian vector :math:`(X_1, X_2, X_3)` with zero mean, unit
# variance and which components :math:`(X_1, X_3)` are correlated.
corr_Matrix = ot.CorrelationMatrix(3)
corr_Matrix[0, 2] = 0.9
distribution = ot.Normal([0] * 3, [1] * 3, corr_Matrix)
sample = distribution.getSample(100)

# %%
# Next, we split the sample in two samples : the first one is associated to  :math:`(X_1, X_2)` and the
# second one is associated to  :math:`X_3`.
first_Sample = sample.getMarginal([0, 1])
second_Sample = sample.getMarginal(2)

# %%
# We fit a linear model of :math:`X_3` with respect to :math:`(X_1, X_2)`:
# :math:`X_3 = a_0 + a_1X_1 + a_2X_2`.
# Then, we test if each coefficient :math:`a_k` is significantly different from 0.
# The null hypothesis is *The coefficient of the linear model is equal to zero*.
# When the result is *True*, the null hypothesis is accepted, which means that
# there is no dependence between the components. When the result is *False*, the null
# hypothesis is rejected, which means that there is a linear relationship between the components.
test_results = ot.LinearModelTest.FullRegression(first_Sample, second_Sample)
for i in range(len(test_results)):
    print(
        "Coefficient a" + str(i) + " is equal to 0?",
        test_results[i].getBinaryQualityMeasure(),
        "p-value=%.6g" % test_results[i].getPValue(),
        "threshold=%.6g" % test_results[i].getThreshold(),
    )

# %%
# **Conclusion**: The test detects the independence between :math:`X_1` and :math:`X_3` and the
# correlation between :math:`X_2` and :math:`X_3`. It also detects that :math:`a_0` is null.



"""
Estimate Sobol' indices for a function with multivariate output
===============================================================
"""

# %%
#
# In this example, we estimate the Sobol' indices of a function by sampling methods.
# This function has several outputs, which leads to the need of aggregated Sobol' indices.
#

# %%
# Introduction
# ------------
#
# In this example we quantify the sensitivity of a function's outputs to its inputs with Sobol' indices.
#
# The function we consider has 5 outputs. In this case, it may be convenient to consider each output separately.
# It may also be interesting to aggregate the sensitivity indices to get a global understanding of the sensitivity of the inputs to the average output.

# %%
# Define the model
# ----------------

# %%
import openturns as ot
import openturns.viewer
import openturns.viewer as otv


# %%
# We define a linear model with 5 independent Gaussian inputs and 2 outputs.

inputDistribution = ot.Normal(5)
function = ot.SymbolicFunction(
    ["x0", "x1", "x2", "x3", "x4"],
    ["x0 + 4.0 * x1 ^ 2 + 3.0 * x2", "-7.0 * x2 - 4.0 * x3 + x4"],
)

# %%
# Estimate the Sobol' indices
# ---------------------------

# %%
# We first create a design of experiments with `SobolIndicesExperiment`.

# %%
size = 1000
sie = ot.SobolIndicesExperiment(inputDistribution, size)
inputDesign = sie.generate()
input_names = inputDistribution.getDescription()
inputDesign.setDescription(input_names)
print("Sample size: ", inputDesign.getSize())

# %%
# We see that 7000 function evaluations are required to estimate the first order and total Sobol' indices.
# Then we evaluate the outputs corresponding to this design of experiments.

# %%
outputDesign = function(inputDesign)

# %%
# Then we estimate the Sobol' indices with the `SaltelliSensitivityAlgorithm`.

# %%
sensitivityAnalysis = ot.SaltelliSensitivityAlgorithm(inputDesign, outputDesign, size)

# %%
# The `getFirstOrderIndices` and `getTotalOrderIndices` method respectively return estimates of first order and total Sobol' indices with a given output.
# Since these depend on the output marginal, the index of the output must
# be specified (the default is to return the index for the first output).

# %%
output_dimension = function.getOutputDimension()
for i in range(output_dimension):
    print("Output #", i)
    first_order = sensitivityAnalysis.getFirstOrderIndices(i)
    total_order = sensitivityAnalysis.getTotalOrderIndices(i)
    print("    First order indices: ", first_order)
    print("    Total order indices: ", total_order)

agg_first_order = sensitivityAnalysis.getAggregatedFirstOrderIndices()
agg_total_order = sensitivityAnalysis.getAggregatedTotalOrderIndices()
print("Agg. first order indices: ", agg_first_order)
print("Agg. total order indices: ", agg_total_order)

# %%
# We see that:
#
# * `x1` has a rather large first order index on the first output, but a small index on the second output,
#
# * `x2` has a rather large first order index on the first output on both outputs,
#
# * the largest aggregated Sobol' index is `x2`,
#
# * `x0` and `x5` have Sobol' indices which are close to zero regardless of whether the indices are aggregated or not.

# %%
# The `draw` method produces the following graph. The vertical bars represent the 95% confidence intervals of the estimates.

# %%
graph = sensitivityAnalysis.draw()
view = otv.View(graph)

# %%
# Since there are several outputs, the graph presents the aggregated Sobol' indices.
# The aggregated Sobol' indices indicate that the input variable which has the largest impact on the variability of the several outputs is `x2`.

# %%
# Display all figures
otv.View.ShowAll()



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

model = ot.SymbolicFunction(["E", "F", "L", "I"], ["F*L^3/(3*E*I)"])

# distribution
# Young's modulus E
E = ot.Beta(0.9, 3.5, 2.5e7, 5.0e7)  # in N/m^2
E.setDescription("E")
# Load F
F = ot.LogNormal()  # in N
F.setParameter(ot.LogNormalMuSigma()([30.0e3, 9e3, 15.0e3]))
F.setDescription("F")
# Length L
L = ot.Uniform(250.0, 260.0)  # in cm
L.setDescription("L")
# Moment of inertia I
II = ot.Beta(2.5, 4, 310, 450)  # in cm^4
II.setDescription("I")
distribution = ot.JointDistribution([E, F, L, II])

# estimator
estimator = ot.SaltelliSensitivityAlgorithm()
estimator.setUseAsymptoticDistribution(True)

# algorithm
algo = ot.SobolSimulationAlgorithm(distribution, model, estimator)
algo.setMaximumOuterSampling(250)  # number of iterations
algo.setExperimentSize(100)
algo.setBlockSize(4)
# alpha: criteria checks whether CIs are small enough
algo.setIndexQuantileLevel(0.05)
# epsilon: criteria checks whether CIs are tight enough
algo.setIndexQuantileEpsilon(1e-2)
algo.run()

graph = algo.drawFirstOrderIndexConvergence(0)

fig = plt.figure(figsize=(8, 4))
axis = fig.add_subplot(111)
axis.set_xlim(auto=True)
otv.View(graph, figure=fig, axes=[axis], add_legend=True)



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

# Create the mesher
mesher = ot.IntervalMesher([10] * 2)

# Create a square
square = ot.Interval([0.0] * 2, [1.0] * 2)

# Mesh the square the usual way
mesh1 = mesher.build(square)

# Mesh the square using diamond mesh
mesh2 = mesher.build(square, True)

# Draw the first mesh
graph1 = mesh1.draw()
graph1.setTitle("diamond=False")
graph1.setXTitle("$x_0$")
graph1.setYTitle("$x_1$")

# Draw the second mesh
graph2 = mesh2.draw()
graph2.setTitle("diamond=True")
graph2.setXTitle("$x_0$")
graph2.setYTitle("$x_1$")

fig = plt.figure(figsize=(10, 4))
graph1_axis = fig.add_subplot(121)
graph2_axis = fig.add_subplot(122)
graph1_axis.set_xlim(auto=True)
graph2_axis.set_xlim(auto=True)

otv.View(graph1, figure=fig, axes=[graph1_axis], add_legend=True)
otv.View(graph2, figure=fig, axes=[graph2_axis], add_legend=True)
fig.suptitle("Mesh of an interval: diamond=False/True")



#! /usr/bin/env python

import openturns as ot

# ot.Log.Show(ot.Log.ALL)

coll = []

# case 1: no transformation
coll.append([ot.Normal(), ot.Normal()])

# case 2: same copula
left = ot.JointDistribution([ot.Normal(), ot.Gumbel()], ot.IndependentCopula(2))
right = ot.JointDistribution([ot.Triangular()] * 2, ot.IndependentCopula(2))
coll.append([left, right])

# case 3: same standard space
left = ot.JointDistribution([ot.Normal(), ot.Gumbel()], ot.IndependentCopula(2))
right = ot.JointDistribution([ot.Triangular()] * 2, ot.GumbelCopula())
coll.append([left, right])

# TODO case 4: different standard space

for left, right in coll:
    transformation = ot.DistributionTransformation(left, right)
    print("left=", left)
    print("right=", right)
    print("transformation=", transformation)
    inverseTransformation = transformation.inverse()
    print("inverseTransformation=", inverseTransformation)
    print("-" * 100)

# with marginaltransformation
copula = ot.BlockIndependentCopula([ot.IndependentCopula(2), ot.IndependentCopula(2)])
distribution = ot.JointDistribution([ot.Normal()] * 4, copula)
print(
    ot.DistributionTransformation(
        ot.IndependentCopula(4), distribution.getMarginal([3, 1, 2, 0])
    )
)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m

ot.TESTPREAMBLE()


# First 8 points of Faure sequence in dimension 1
expected = ot.Sample(
    [
        [1.0 / 2.0],
        [1.0 / 4.0],
        [3.0 / 4.0],
        [1.0 / 8.0],
        [5.0 / 8.0],
        [3.0 / 8.0],
        [7.0 / 8.0],
        [1.0 / 16.0],
    ]
)
sequence = ot.FaureSequence(1)
print(sequence)
faureSample = sequence.generate(8)
ott.assert_almost_equal(faureSample, expected)

# First 8 points of Faure sequence in dimension 2
expected = ot.Sample(
    [
        [0.5, 0.5],
        [0.25, 0.75],
        [0.75, 0.25],
        [0.125, 0.625],
        [0.625, 0.125],
        [0.375, 0.375],
        [0.875, 0.875],
        [0.0625, 0.9375],
    ]
)
sequence = ot.FaureSequence(2)
print(sequence)
faureSample = sequence.generate(8)
ott.assert_almost_equal(faureSample, expected)

# First 8 points of Faure sequence in dimension 3
expected = ot.Sample(
    [
        [1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0],
        [2.0 / 3.0, 2.0 / 3.0, 2.0 / 3.0],
        [1.0 / 9.0, 4.0 / 9.0, 7.0 / 9.0],
        [4.0 / 9.0, 7.0 / 9.0, 1.0 / 9.0],
        [7.0 / 9.0, 1.0 / 9.0, 4.0 / 9.0],
        [2.0 / 9.0, 8.0 / 9.0, 5.0 / 9.0],
        [5.0 / 9.0, 2.0 / 9.0, 8.0 / 9.0],
        [8.0 / 9.0, 5.0 / 9.0, 2.0 / 9.0],
    ]
)
sequence = ot.FaureSequence(3)
print(sequence)
faureSample = sequence.generate(8)
ott.assert_almost_equal(faureSample, expected)

# Create a Faure sequence in dimension 15
sequence = ot.FaureSequence(15)
print(sequence)
faureSample = sequence.generate(10)

# Create another Faure' sequence of dimension 2 to estimate Pi in [0 1)^2
dimension = 2
sequence = ot.FaureSequence(dimension)
pointInsideCircle = 0
sampleSize = 3**7  # This is significant!
for i in range(sampleSize):
    faurePoint = sequence.generate()
    if faurePoint.norm() < 1.0:
        pointInsideCircle += 1
probabilityEstimate = (1.0 * pointInsideCircle) / sampleSize
probability = m.pi / 4.0
print("sample size=", sampleSize)
print("computed probability =", probabilityEstimate)
print("expected probability =", probability)
rtol = 10.0 / sampleSize
ott.assert_almost_equal(probability, probabilityEstimate, rtol)

# Test against GSobol' test function
# https://github.com/openturns/openturns/issues/2653
a = [0.0, 9.0, 99.0]


def GSobolModel(X):
    X = ot.Point(X)
    d = X.getDimension()
    Y = 1.0
    for i in range(d):
        Y *= (abs(4.0 * X[i] - 2.0) + a[i]) / (1.0 + a[i])
    return ot.Point([Y])


dimension = 3
gSobolFunction = ot.PythonFunction(dimension, 1, GSobolModel)
gSobolFunction.setOutputDescription(["Y"])

# Define the distribution
distributionList = [ot.Uniform(0.0, 1.0) for i in range(dimension)]
distribution = ot.JointDistribution(distributionList)

# Compute Mean
gSobolMean = 1.0
basis = 3  # ie the smallest prime number greater or equal to the dimension
sampleSize = basis**7  # This is significant!
sequence = ot.FaureSequence()
experiment_QMC = ot.LowDiscrepancyExperiment(sequence, distribution, sampleSize, False)
inputSample = experiment_QMC.generate()
outputSample = gSobolFunction(inputSample)
computedMean = outputSample.computeMean()[0]
print("sample size=", sampleSize)
print("computed mean =", computedMean)
print("expected mean =", gSobolMean)
rtol = 10.0 / sampleSize
ott.assert_almost_equal(computedMean, gSobolMean, rtol)



"""
Quick start:  the Ishigami function
===================================
"""

# %%
#
# In this example, we create a polynomial chaos for the
# :ref:`Ishigami function<use-case-ishigami>`.
# We create a sparse polynomial with maximum total degree equal to 8
# using the :class:`~openturns.FunctionalChaosAlgorithm` class.
#

# %%
# Define the model
# ----------------

# %%
from openturns.usecases import ishigami_function
import openturns as ot
import openturns.viewer as otv
import math

# %%
# We load the Ishigami model :
im = ishigami_function.IshigamiModel()

# %%
# The `IshigamiModel` data class contains the input distribution :math:`\vect{X}=(X_1, X_2, X_3)` in `im.distribution` and the Ishigami function in `im.model`.
# We also have access to the input variable names with
input_names = im.distribution.getDescription()


# %%
# Draw the function
# -----------------

# %%
# Create a training sample

# %%
sampleSize = 1000
inputSample = im.distribution.getSample(sampleSize)
outputSample = im.model(inputSample)


# %%
# Display relationships between the output and the inputs

# %%
grid = ot.VisualTest.DrawPairsXY(inputSample, outputSample)
view = otv.View(grid, figure_kw={"figsize": (12.0, 4.0)})

# %%
graph = ot.HistogramFactory().build(outputSample).drawPDF()
view = otv.View(graph)

# %%
# We see that the distribution of the output has two modes.

# %%
# Create the polynomial chaos model
# ---------------------------------

# %%
# Create a training sample

# %%
sampleSize = 100
inputTrain = im.distribution.getSample(sampleSize)
outputTrain = im.model(inputTrain)

# %%
# Create the chaos.
#
# We could use only the input and output training samples: in this case, the distribution of the input sample is computed by selecting the distribution that has the best fit.

# %%
chaosalgo = ot.FunctionalChaosAlgorithm(inputTrain, outputTrain)

# %%
# Since the input distribution is known in our particular case, we instead create
# the multivariate basis from the distribution.
# We define the orthogonal basis used to expand the function.
# We see that each input has the uniform distribution, which corresponds to the
# Legendre polynomials.

# %%
multivariateBasis = ot.OrthogonalProductPolynomialFactory([im.X1, im.X2, im.X3])
multivariateBasis

# %%
# Then we create the sparse polynomial chaos expansion using regression and
# the LARS selection model method.

# %%
selectionAlgorithm = ot.LeastSquaresMetaModelSelectionFactory()
projectionStrategy = ot.LeastSquaresStrategy(selectionAlgorithm)
totalDegree = 8
enumerateFunction = multivariateBasis.getEnumerateFunction()
basisSize = enumerateFunction.getBasisSizeFromTotalDegree(totalDegree)
adaptiveStrategy = ot.FixedStrategy(multivariateBasis, basisSize)
chaosAlgo = ot.FunctionalChaosAlgorithm(
    inputTrain, outputTrain, im.distribution, adaptiveStrategy, projectionStrategy
)

# %%
# The coefficients of the polynomial expansion can then be estimated
# using the :meth:`~openturns.FunctionalChaosAlgorithm.run` method.

# %%
chaosAlgo.run()

# %%
# The :meth:`~openturns.FunctionalChaosAlgorithm.getResult` method returns the
# result.
chaosResult = chaosAlgo.getResult()

# %%
# The polynomial chaos result provides a pretty-print.

# %%
chaosResult

# %%
# Get the metamodel.

# %%
metamodel = chaosResult.getMetaModel()

# %%
# In order to validate the metamodel, we generate a test sample.

# %%
n_valid = 1000
inputTest = im.distribution.getSample(n_valid)
outputTest = im.model(inputTest)
metamodelPredictions = metamodel(inputTest)
val = ot.MetaModelValidation(outputTest, metamodelPredictions)
r2Score = val.computeR2Score()[0]
r2Score

# %%
# The :math:`R^2` is very close to 1: the metamodel is accurate.

# %%
graph = val.drawValidation()
graph.setTitle("R2=%.2f%%" % (r2Score * 100))
view = otv.View(graph)

# %%
# The metamodel has a good predictivity, since the points are almost on the first diagonal.

# %%
# Compute mean and variance
# -------------------------

# %%
# The mean and variance of a polynomial chaos expansion are computed
# using the coefficients of the expansion.
# This can be done using :class:`~openturns.FunctionalChaosRandomVector`.

# %%
randomVector = ot.FunctionalChaosRandomVector(chaosResult)
mean = randomVector.getMean()
print("Mean=", mean)
covarianceMatrix = randomVector.getCovariance()
print("Covariance=", covarianceMatrix)
outputDimension = outputTrain.getDimension()
stdDev = ot.Point(outputDimension)
for i in range(outputDimension):
    stdDev[i] = math.sqrt(covarianceMatrix[i, i])
print("Standard deviation=", stdDev)

# %%
# Compute and print Sobol' indices
# --------------------------------

# %%
# By default, printing the object will print the Sobol' indices
# and the multi-indices ordered by decreasing part of variance.
# If a multi-index has a part of variance which is lower
# than some threshold, it is not printed.
# This threshold can be customized using the
# `FunctionalChaosSobolIndices-VariancePartThreshold` key of the
# :class:`~openturns.ResourceMap`.

# %%
chaosSI = ot.FunctionalChaosSobolIndices(chaosResult)
chaosSI

# %%
# We notice that a coefficient with marginal degree equal to 6 has a significant impact on the output variance.
# Hence, we cannot get a satisfactory polynomial chaos with total degree less that 6.

# %%
# Draw Sobol' indices.

# %%
dim_input = im.distribution.getDimension()
first_order = [chaosSI.getSobolIndex(i) for i in range(dim_input)]
total_order = [chaosSI.getSobolTotalIndex(i) for i in range(dim_input)]
input_names = im.model.getInputDescription()
graph = ot.SobolIndicesAlgorithm.DrawSobolIndices(input_names, first_order, total_order)
view = otv.View(graph)


# %%
# The variable which has the largest impact on the output is, taking
# interactions into account, :math:`X_1`.
# We see that :math:`X_1` has interactions with other variables, since the first
# order indice is less than the total order indice.
# At first order, :math:`X_3` has no interaction with other variables since its
# first order indice is close to zero.

# %%
# Computing the accuracy
# ----------------------

# %%
# The interesting point with the Ishigami function is that the exact Sobol' indices are known.
# We can use that property in order to compute the absolute error on the Sobol' indices for the polynomial chaos.

# %%
# To make the comparisons simpler, we gather the results into a list.

# %%
S_exact = [im.S1, im.S2, im.S3]
ST_exact = [im.ST1, im.ST2, im.ST3]

# %%
# Then we perform a loop over the input dimension and compute the absolute error on the Sobol' indices.

# %%
for i in range(im.dim):
    absoluteErrorS = abs(first_order[i] - S_exact[i])
    absoluteErrorST = abs(total_order[i] - ST_exact[i])
    print(
        "X%d, Abs.Err. on S=%.1e, Abs.Err. on ST=%.1e"
        % (i + 1, absoluteErrorS, absoluteErrorST)
    )

# %%
# We see that the indices are correctly estimated with a low accuracy even if we have used only 100 function evaluations.
# This shows the good performance of the polynomial chaos in this case.

# %%
# We can compute the part of the variance of the output explained by each multi-index.

# %%
partOfVariance = chaosSI.getPartOfVariance()
chaosResult = chaosSI.getFunctionalChaosResult()
coefficients = chaosResult.getCoefficients()
orthogonalBasis = chaosResult.getOrthogonalBasis()
enumerateFunction = orthogonalBasis.getEnumerateFunction()
indices = chaosResult.getIndices()
basisSize = indices.getSize()
print("Index, global index, multi-index, coefficient, part of variance")
for i in range(basisSize):
    globalIndex = indices[i]
    multiIndex = enumerateFunction(globalIndex)
    if partOfVariance[i] > 1.0e-3:
        print(
            "%d, %d, %s, %.4f, %.4f"
            % (i, globalIndex, multiIndex, coefficients[i, 0], partOfVariance[i])
        )

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import math as m

ot.TESTPREAMBLE()

f = ot.SymbolicFunction(["t", "y0", "y1"], ["t - y0", "y1 + t^2"])
nt = 100
timeGrid = [(i**2.0) / (nt - 1.0) ** 2.0 for i in range(nt)]
phi = ot.ParametricFunction(f, [0], [0.0])
solver = ot.Fehlberg(phi)
print("ODE solver=", solver)
initialState = [1.0, -1.0]
print("time grid=", ot.Point(timeGrid))
result = solver.solve(initialState, timeGrid)
print("result=", result)
print("last value=", result[nt - 1])
t = timeGrid[nt - 1]
ref = ot.Point(2)
ref[0] = -1.0 + t + 2.0 * m.exp(-t)
ref[1] = -2.0 + -2.0 * t - t * t + m.exp(t)
print("ref. value=", ref)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.Dirac(0.7)
size = 10000
sample = distribution.getSample(size)
factory = ot.DiracFactory()
estimatedDistribution = factory.build(sample)
print("Distribution          =", distribution)
print("Estimated distribution=", estimatedDistribution)
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedDirac = factory.buildAsDirac(sample)
print("Dirac          =", distribution)
print("Estimated dirac=", estimatedDirac)
estimatedDirac = factory.buildAsDirac()
print("Default dirac=", estimatedDirac)
estimatedDirac = factory.buildAsDirac(distribution.getParameter())
print("Dirac from parameters=", estimatedDirac)



#! /usr/bin/env python

import openturns as ot

ot.PlatformInfo.SetNumericalPrecision(6)

dim = 2

# First domain: [0,2]x[0,2]
cube = ot.Interval([0.0] * dim, [2.0] * dim)

# Second domain: sphere center=(0,0) r=1
function = ot.SymbolicFunction(["x", "y"], ["x^2 + y^2"])
sphere = ot.LevelSet(function, ot.LessOrEqual(), 1.0)

# Inside sphere but not cube
p0 = [-0.25, 0.25]
# Inside cube and sphere
p1 = [0.25, 0.25]
# Inside cube but not sphere
p2 = [1.8, 1.8]
# Outside
p3 = [4.0, 4.0]

domain = ot.DomainDisjunctiveUnion([cube, sphere])
print("cube=", cube)
print("sphere=", sphere)
print("disjunctive union=", domain)
# Accessors
print("Dimension=", domain.getDimension())
# Contains
print("is point ", p0, " inside ? ", domain.contains(p0))
print("is point ", p1, " inside ? ", domain.contains(p1))
print("is point ", p2, " inside ? ", domain.contains(p2))
print("is point ", p3, " inside ? ", domain.contains(p3))
sample = [p0, p1, p2, p3]
print("is sample ", sample, " inside ? ", domain.contains(sample))

# p1 is inside cube & sphere, no need to check sphere2
sphere2 = ot.LevelSet(ot.SymbolicFunction(["x", "y"], ["x/0"]), ot.Less(), 1.0)
domain2 = ot.DomainDisjunctiveUnion([cube, sphere, sphere2])
assert not domain2.contains(p1), "prune sphere"

# computeDistance
interval1 = ot.Interval(-0.4, 0.0)
interval2 = ot.Interval(1.0, 1.3)
interval3 = ot.Interval(1.1, 1.5)
domain = ot.DomainDisjunctiveUnion([interval1, interval2, interval3])
x_dref = {-1.0: 0.6, -0.25: 0.0, 0.875: 0.125, 1.05: 0.0, 1.2: None, 2.0: 0.5}
for x, dref in x_dref.items():
    try:
        d = domain.computeDistance([x])
    except RuntimeError:
        d = None
    print(f"x={x} d={d} ref={dref}")
    if dref is not None:
        assert abs(d - dref) < 1e-8, "distance"



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.FarlieGumbelMorgensternCopula(0.7)
size = 1000
sample = distribution.getSample(size)
factory = ot.FarlieGumbelMorgensternCopulaFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedFarlieGumbelMorgensternCopula = factory.buildAsFarlieGumbelMorgensternCopula(
    sample
)
print("FarlieGumbelMorgensternCopula          =", distribution)
print(
    "Estimated farlieGumbelMorgensternCopula=", estimatedFarlieGumbelMorgensternCopula
)
estimatedFarlieGumbelMorgensternCopula = factory.buildAsFarlieGumbelMorgensternCopula()
print("Default farlieGumbelMorgensternCopula=", estimatedFarlieGumbelMorgensternCopula)
estimatedFarlieGumbelMorgensternCopula = factory.buildAsFarlieGumbelMorgensternCopula(
    distribution.getParameter()
)
print(
    "FarlieGumbelMorgensternCopula from parameters=",
    estimatedFarlieGumbelMorgensternCopula,
)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
from math import sqrt

ot.PlatformInfo.SetNumericalPrecision(5)
distribution = ot.FisherSnedecor(4.5, 8.4)
size = 10000
sample = distribution.getSample(size)
factory = ot.FisherSnedecorFactory()
print("Distribution                      =", repr(distribution))
result = factory.buildEstimator(sample)
estimatedDistribution = result.getDistribution()
print("Estimated distribution            =", repr(estimatedDistribution))
parameterDistribution = result.getParameterDistribution()
print("Parameter distribution            =", parameterDistribution)
defaultDistribution = factory.build()
print("Default distribution              =", defaultDistribution)
fromParameterDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters      =", fromParameterDistribution)
typedEstimatedDistribution = factory.buildAsFisherSnedecor(sample)
print("Typed estimated distribution      =", typedEstimatedDistribution)
defaultTypedDistribution = factory.buildAsFisherSnedecor()
print("Default typed distribution        =", defaultTypedDistribution)
typedFromParameterDistribution = factory.buildAsFisherSnedecor(
    distribution.getParameter()
)
print("Typed distribution from parameters=", typedFromParameterDistribution)
# Various estimators
estimatedDistribution = factory.build(sample)
print("Estimated distribution with default estimator =", estimatedDistribution)
estimatedDistribution = factory.buildMethodOfMoments(sample)
print("Estimated distribution with moments=", estimatedDistribution)
estimatedDistribution = factory.buildMethodOfLikelihoodMaximization(sample)
print("Estimated distribution with likelihoodMax.=", estimatedDistribution)

# Build method of moments
print("Build method of moments")
size = 10000
distribution = ot.FisherSnedecor(4.5, 8.4)
sample = distribution.getSample(size)
factory = ot.FisherSnedecorFactory()
estimatedTN = factory.buildMethodOfMoments(sample)
exact_mu = sample.computeMean()[0]
exact_sigma2 = sample.computeCovariance()[0, 0]
computed_mu = estimatedTN.getMean()[0]
computed_sigma2 = estimatedTN.getCovariance()[0, 0]
ott.assert_almost_equal(exact_mu, computed_mu, 1.0e-15, 0.0)
ott.assert_almost_equal(exact_sigma2, computed_sigma2, 1.0e-15, 0.0)

# Build method of likelihood maximization
print("Build method of likelihood maximization")
size = 10000
distribution = ot.FisherSnedecor(4.5, 8.4)
sample = distribution.getSample(size)
factory = ot.FisherSnedecorFactory()
estimatedTN = factory.buildMethodOfLikelihoodMaximization(sample)
exact_d1 = distribution.getD1()
exact_d2 = distribution.getD2()
computed_d1 = estimatedTN.getD1()
computed_d2 = estimatedTN.getD2()
ott.assert_almost_equal(exact_d1, computed_d1, 0.0, 100.0 / sqrt(size))
ott.assert_almost_equal(exact_d2, computed_d2, 0.0, 100.0 / sqrt(size))



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# need a proper LS solver
if not ot.PlatformInfo.HasFeature("cminpack") or not ot.PlatformInfo.HasFeature(
    "ceres"
):
    exit(0)

size = 10000

distribution = ot.Gumbel(1.5, -0.5)
print("distribution=", distribution)
sample = distribution.getSample(size)
factory = ot.MethodOfMomentsFactory(ot.Gumbel(), [1, 2])
inf_dist = factory.build(sample)
print("estimated distribution=", inf_dist)
p_ref = [1.50988, -0.481183]
ott.assert_almost_equal(inf_dist.getParameter(), p_ref, 1e-2, 1e-2)

# set (a,b) out of (r, t, a, b)
distribution = ot.Beta(2.3, 2.2, -1.0, 1.0)
print("distribution=", distribution)
sample = distribution.getSample(size)
factory = ot.MethodOfMomentsFactory(ot.Beta(), [1, 2])
factory.setKnownParameter([2, 3], [-1.0, 1.0])
inf_dist = factory.build(sample)
print("estimated distribution=", inf_dist)
p_ref = [2.27806, 2.20053, -1, 1]
ott.assert_almost_equal(inf_dist.getParameter(), p_ref, 1e-2, 1e-2)

# from moments
distribution = ot.Beta(2.3, 2.2, -1.0, 1.0)
factory = ot.MethodOfMomentsFactory(ot.Beta(), [1, 2, 3, 4])
cm = [distribution.getCentralMoment(i + 2)[0] for i in range(3)]
moments = [distribution.getMean()[0]] + cm
inf_dist = factory.buildFromMoments(moments)
print("estimated distribution (moments)=", inf_dist)
p_ref = [2.3, 2.2, -1, 1]
ott.assert_almost_equal(inf_dist.getParameter(), p_ref, 1e-2, 1e-2)

# with bounds
data = [
    0.6852,
    0.9349,
    0.5884,
    1.727,
    1.581,
    0.3193,
    -0.5701,
    1.623,
    2.210,
    -0.3440,
    -0.1646,
]
sample = ot.Sample([[x] for x in data])
size = sample.getSize()
xMin = sample.getMin()[0]
xMax = sample.getMax()[0]
delta = xMax - xMin
a = xMin - delta / (size + 2)
b = xMax + delta / (size + 2)
distribution = ot.TruncatedNormal()
factory = ot.MethodOfMomentsFactory(distribution, [1, 2, 3, 4])
factory.setKnownParameter([2, 3], [a, b])
solver = factory.getOptimizationAlgorithm()
sampleMean = sample.computeMean()[0]
sampleSigma = sample.computeStandardDeviation()[0]
startingPoint = [sampleMean, sampleSigma]
solver.setStartingPoint(startingPoint)
factory.setOptimizationAlgorithm(solver)
lowerBound = [-1.0, 0]
upperBound = [-1.0, 1.5]
finiteLowerBound = [False, True]
finiteUpperBound = [False, True]
bounds = ot.Interval(lowerBound, upperBound, finiteLowerBound, finiteUpperBound)
factory = ot.MethodOfMomentsFactory(distribution, [3, 4], bounds)
factory.setKnownParameter([2, 3], [a, b])
factory.setOptimizationBounds(bounds)
inf_dist = factory.build(sample)
print("estimated distribution=", inf_dist)
p_ref = [0.805158, 1.5, -0.783954, 2.42385]
ott.assert_almost_equal(inf_dist.getParameter(), p_ref, 1e-2, 1e-2)

# setKnownParameter+buildEstimator
sample = ot.Normal(2.0, 1.0).getSample(size)
factory = ot.MethodOfMomentsFactory(ot.Normal(), [1])
factory.setBootstrapSize(4)
factory.setKnownParameter([1], [1.0])  # set the sigma parameter to 1.0
result = factory.buildEstimator(sample)
inf_dist = result.getDistribution().getParameter()
print("estimated distribution=", inf_dist)
p_ref = [2.04553, 1.0]
ott.assert_almost_equal(result.getDistribution().getParameter(), p_ref, 1e-3, 1e-3)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


def cleanScalar(inScalar):
    if abs(inScalar) < 1.0e-6:
        return 0.0
    return inScalar


# Default constructor
myDefaultModel = ot.UserDefinedSpectralModel()
print("myDefaultModel = ", myDefaultModel)

# Default dimension parameter to evaluate the model
dimension = 2
inputDimension = 1

# Amplitude values
amplitude = ot.Point(dimension)
# Scale values
scale = ot.Point(inputDimension, 1.0)
# Spatial correlation
spatialCorrelation = ot.CorrelationMatrix(dimension)
for index in range(dimension):
    # constant amplitude
    amplitude[index] = (index + 1.0) / dimension
    if index > 0:
        spatialCorrelation[index, index - 1] = 1.0 / index

# Sample a CauchyModel
referenceModel = ot.CauchyModel(scale, amplitude, spatialCorrelation)

size = 5
frequencyGrid = ot.RegularGrid(0.0, 2.0 / size, size)
dspCollection = ot.HermitianMatrixCollection(frequencyGrid.getN())
for i in range(frequencyGrid.getN()):
    dspCollection[i] = referenceModel(frequencyGrid.getValue(i))

# Create a UserDefinedSpectralModel
myModel = ot.UserDefinedSpectralModel(frequencyGrid, dspCollection)
print("myModel=", myModel)

# Sample the UserDefinedSpectralModel
samplingGrid = ot.RegularGrid(-0.4, 1.0 / 16, 5 * size)

for i in range(samplingGrid.getN()):
    frequency = samplingGrid.getValue(i)
    print("frequency= %g myModel=" % cleanScalar(frequency))
    print(myModel(frequency).clean(1e-6), ", referenceModel=")
    print(referenceModel(frequency).clean(1e-6))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

dim = 2

x = [2.0 + i for i in range(dim)]

print("x=", x)

factory = ot.ConstantBasisFactory(dim)
print("factory=", factory)
basis = factory.build()
print("basis=", basis)

f = ot.AggregatedFunction(basis)
y = f(x)
print("y=", y)

factory = ot.LinearBasisFactory(dim)
print("factory=", factory)
basis = factory.build()
print("basis=", basis)

f = ot.AggregatedFunction(basis)
y = f(x)
print("y=", y)

factory = ot.QuadraticBasisFactory(dim)
print("factory=", factory)
basis = factory.build()
print("basis=", basis)

f = ot.AggregatedFunction(basis)
y = f(x)
print("y=", y)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

# Defining parameters
dimension = 3
bounds = ot.Interval(dimension)
size = 25

# Build standard LHS algorithm
distribution = ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension)
distribution.setDescription(["U" + str(i) for i in range(dimension)])
lhs = ot.LHSExperiment(distribution, size)
lhs.setRandomShift(False)  # centered
lhs.setAlwaysShuffle(True)  # randomized

# print the object
print("lhs=", lhs)
print("Bounds of uniform distributions=", distribution.getRange())

# Generate design without optimization
design = lhs.generate()
print("design=", design)

# Defining space fillings
spaceFillingC2 = ot.SpaceFillingC2()
spaceFillingPhiP = ot.SpaceFillingPhiP(10)
spaceFillingMinDist = ot.SpaceFillingMinDist()

# print the criteria on this design
print(
    "PhiP=%f, C2=%f"
    % (ot.SpaceFillingPhiP().evaluate(design), ot.SpaceFillingC2().evaluate(design))
)

# --------------------------------------------------#
# ------------- Simulated annealing  ------------- #
# --------------------------------------------------#
# Geometric profile
T0 = 10.0
iMax = 2000
c = 0.95
geomProfile = ot.GeometricProfile(T0, c, iMax)

# 1) Simulated Annealing LHS with geometric temperature profile, C2
# optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingC2, geomProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print("Generating design using geometric temperature profile & C2 criterion=", design)
result = optimalLHSAlgorithm.getResult()
print(
    "Final criteria: C2=%f, PhiP=%f, MinDist=%f"
    % (result.getC2(), result.getPhiP(), result.getMinDist())
)

# Linear profile
linearProfile = ot.LinearProfile(T0, iMax)

# 2) Simulated Annealing LHS with linear temperature profile, PhiP optimization
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(lhs, spaceFillingPhiP, linearProfile)
print("lhs=", optimalLHSAlgorithm)
design = optimalLHSAlgorithm.generate()
print("Generating design using linear temperature profile & PhiP criterion =", design)
result = optimalLHSAlgorithm.getResult()
print(
    "Final criteria: C2=%f, PhiP=%f, MinDist=%f"
    % (result.getC2(), result.getPhiP(), result.getMinDist())
)

# 3) Simulated Annealing LHS with geometric temperature profile, PhiP
# optimization & initial design
initialDesign = ot.Sample(design)
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(
    initialDesign, distribution, spaceFillingPhiP, geomProfile
)
print("lhs=", optimalLHSAlgorithm)
print("initial design=", initialDesign)
print(
    "PhiP=%f, C2=%f"
    % (ot.SpaceFillingPhiP().evaluate(design), ot.SpaceFillingC2().evaluate(design))
)
design = optimalLHSAlgorithm.generate()
print("Generating design using linear temperature profile & PhiP criterion =", design)
result = optimalLHSAlgorithm.getResult()
print(
    "Final criteria: C2=%f, PhiP=%f, MinDist=%f"
    % (result.getC2(), result.getPhiP(), result.getMinDist())
)
# 4) Simulated Annealing LHS with linear temperature profile, PhiP
# optimization and nStart > 1
nStart = 10
design = optimalLHSAlgorithm.generateWithRestart(nStart)
print("Generating design using linear temperature profile & PhiP criterion =", design)
results = optimalLHSAlgorithm.getResult()
print(
    "Final criteria: C2=%f, PhiP=%f, MinDist=%f"
    % (results.getC2(), results.getPhiP(), results.getMinDist())
)
for i in range(nStart):
    design = results.getOptimalDesign(i)
    print("  Intermediate design for restart iteration number ", i, design)
    print(
        "  Final criteria: C2=%f, PhiP=%f, MinDist=%f"
        % (results.getC2(i), results.getPhiP(i), results.getMinDist(i))
    )

# 5) Fix https://github.com/openturns/openturns/issues/1826
optimalLHSAlgorithm = ot.SimulatedAnnealingLHS(
    initialDesign, distribution, spaceFillingMinDist, geomProfile
)
design = optimalLHSAlgorithm.generate()
result = optimalLHSAlgorithm.getResult()
# optim ok
# Final MinDist is >= initial one
optimal_design = [
    [0.58, 0.3, 0.18],
    [0.9, 0.38, 0.3],
    [0.38, 0.74, 0.66],
    [0.06, 0.5, 0.26],
    [0.34, 0.98, 0.5],
    [0.26, 0.7, 0.14],
    [0.74, 0.34, 0.62],
    [0.82, 0.66, 0.58],
    [0.78, 0.06, 0.1],
    [0.54, 0.94, 0.06],
    [0.46, 0.22, 0.54],
    [0.1, 0.78, 0.46],
    [0.94, 0.02, 0.82],
    [0.5, 0.26, 0.86],
    [0.42, 0.58, 0.42],
    [0.22, 0.18, 0.34],
    [0.18, 0.54, 0.94],
    [0.14, 0.14, 0.74],
    [0.7, 0.46, 0.98],
    [0.66, 0.86, 0.78],
    [0.98, 0.1, 0.38],
    [0.62, 0.62, 0.02],
    [0.3, 0.42, 0.7],
    [0.02, 0.9, 0.9],
    [0.86, 0.82, 0.22],
]

assert result.getMinDist() >= ot.SpaceFillingMinDist().evaluate(initialDesign)
assert result.getC2() <= ot.SpaceFillingMinDist().evaluate(initialDesign)
assert result.getPhiP() <= ot.SpaceFillingPhiP().evaluate(initialDesign)
ott.assert_almost_equal(result.getC2(), 0.05473015652160929)
ott.assert_almost_equal(result.getPhiP(), 3.517772966753692)
ott.assert_almost_equal(result.getMinDist(), 0.29120439557122074)
ott.assert_almost_equal(result.getOptimalDesign(), optimal_design)



"""
Visualize clouds
================
"""

# %%
# In this example we are going to draw clouds of points from a data sample.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create 2-d samples to visualize
N = 500
R = ot.CorrelationMatrix(2)
R[0, 1] = -0.7
# 2d N(1,1) with correlation
sample1 = ot.Normal([1.0] * 2, [1.0] * 2, R).getSample(N)
sample2 = ot.Normal(2).getSample(N)  # 2d N(0,1) independent

# %%
# Create cloud drawables
cloud1 = ot.Cloud(sample1, "blue", "fsquare", "First Cloud")
cloud2 = ot.Cloud(sample2, "red", "fsquare", "Second Cloud")

# Then, assemble it into a graph
myGraph2d = ot.Graph("2d clouds", "x1", "x2")
myGraph2d.setLegendPosition("upper right")
myGraph2d.add(cloud1)
myGraph2d.add(cloud2)
view = otv.View(myGraph2d)

# %%
# Create a 3-d sample
mean = [0.0] * 3
sigma = [2.0, 1.5, 1.0]
R = ot.CorrelationMatrix(3)
R[0, 1] = 0.8
R[1, 2] = -0.5
N = 500
sample3 = ot.Normal(mean, sigma, R).getSample(N)

# %%
# Draw clouds pairs
graph3 = ot.VisualTest.DrawPairs(sample3)
graph3.setTitle("3d clouds")
view = otv.View(graph3)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

legendre = ot.LegendreFactory()
print("legendre=", legendre)
for i in range(10):
    print("legendre(", i, ")=", legendre.build(i))
roots = legendre.getRoots(10)
print("legendre(10) roots=", repr(roots))
nodes, weights = legendre.getNodesAndWeights(10)
print("legendre(10) nodes=", nodes, "and weights=", weights)



#! /usr/bin/env python

import openturns as ot

# Bounds are [0,1]^dimension
dimension = 2

# Size of sample
size = 20

# Factory: lhs generates
lhsDesign = ot.LHSExperiment(
    ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension), size
)
lhsDesign.setRandomShift(False)  # centered
lhsDesign.setAlwaysShuffle(True)  # randomized

# For each design, evaluation of some criteria
fp = open("space_filling_criteria_perturbation.val", "w")
design = lhsDesign.generate()
filename = "design_dim_2_size_20_centered_perturbLHS.csv"
design.exportToCSVFile(filename, ";")
c2 = ot.SpaceFillingC2().evaluate(design)
phip = ot.SpaceFillingPhiP().evaluate(design)
mindist = ot.SpaceFillingMinDist().evaluate(design)
fp.write("initial design=%s\n" % filename)
fp.write("c2=%1.10e, phip=%1.10e, mindist=%1.10e\n" % (c2, phip, mindist))

# Perturbations
# Echange of type:
# tmp = design[row1, column]
# design[row1, column] = design[row2, column]
# design[row2, column] = tmp
for row1 in range(size):
    for row2 in range(size):
        for column in range(dimension):
            # The 3-upplet (row1, row2, column)
            # Criteria exchange design[row1, column] and design[row2, column]
            updated_c2 = ot.SpaceFillingC2().perturbLHS(design, c2, row1, row2, column)
            # updated_phip = ot.SpaceFillingPhiP().perturbLHS(design, phip, row1, row2, column)
            fp.write(
                "row1=%d, row2=%d, column=%d, updated c2=%1.10e\n"
                % (row1 + 1, row2 + 1, column + 1, updated_c2)
            )

# New test with biggest dimension
dimension = 50

# Size of sample
size = 100
lhsDesign = ot.LHSExperiment(
    ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension), size
)
lhsDesign.setRandomShift(False)  # centered
lhsDesign.setAlwaysShuffle(True)  # randomized

design = lhsDesign.generate()
# export the design
filename = "design_dim_50_size_100_centered_perturbLHS.csv"
design.exportToCSVFile(filename, ";")
c2 = ot.SpaceFillingC2().evaluate(design)
phip = ot.SpaceFillingPhiP().evaluate(design)
mindist = ot.SpaceFillingMinDist().evaluate(design)
fp.write("initial design=%s\n" % filename)
fp.write("c2=%1.10e, phip=%1.10e, mindist=%1.10e\n" % (c2, phip, mindist))
for row1 in range(0, size, 10):
    for row2 in range(0, size, 10):
        for column in range(0, dimension, 10):
            # The 3-upplet (row1, row2, column)
            # Criteria exchange design[row1, column] and design[row2, column]
            updated_c2 = ot.SpaceFillingC2().perturbLHS(design, c2, row1, row2, column)
            updated_phip = ot.SpaceFillingPhiP().perturbLHS(
                design, phip, row1, row2, column
            )
            fp.write(
                "row1=%d, row2=%d, column=%d, updated c2=%1.10e\n"
                % (row1 + 1, row2 + 1, column + 1, updated_c2)
            )

fp.close()



"""
Create a Gauss product design
=============================
"""

# %%
# In this example we are going to create a deterministic weighted design experiment using Gauss product.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Define the underlying distribution, degrees
distribution = ot.JointDistribution([ot.Exponential(), ot.Triangular(-1.0, -0.5, 1.0)])
marginalSizes = [15, 8]

# %%
# Create the design
experiment = ot.GaussProductExperiment(distribution, marginalSizes)
sample = experiment.generate()

# %%
# Plot the design
graph = ot.Graph("GP design", "x1", "x2")
cloud = ot.Cloud(sample, "blue", "fsquare", "")
graph.add(cloud)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

size = 100
dim = 2

sampleX = ot.Normal(dim).getSample(size)
sampleX0 = sampleX.getMarginal(0)
sampleY = sampleX.getMarginal(1)
sampleZ = ot.SymbolicFunction("x", "exp(x)")(sampleX0)
# Spearman Test : test if two samples have a zero rank correlation
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

print("Spearman=", ot.HypothesisTest.Spearman(sampleY, sampleZ, 0.10))

# Full Spearman Test : collection of tests of zero correlation between the 1D marginals of a sample and another 1D sample
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

# Expected result is SpecFunc::Infinity
fullSpearman = ot.HypothesisTest.FullSpearman(sampleX, sampleZ, 0.10)[0]
testStatistic = fullSpearman.getStatistic()
assert testStatistic == ot.SpecFunc.Infinity

fullSpearman = ot.HypothesisTest.FullSpearman(sampleX, sampleZ, 0.10)[1]
pValue = fullSpearman.getPValue()
testStatistic = fullSpearman.getStatistic()
ott.assert_almost_equal(pValue, 0.903, 1e-4, 0.0)
ott.assert_almost_equal(testStatistic, 0.1219, 1e-4, 0.0)

# Expected result is SpecFunc::Infinity
fullSpearman = ot.HypothesisTest.FullSpearman(sampleY, sampleY, 0.10)[0]
testStatistic = fullSpearman.getStatistic()
assert testStatistic == ot.SpecFunc.Infinity

# Partial Spearman Test : collection of tests of zero correlation between a selection of the 1D marginals of a sample and another 1D sample
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

selection = [0]
# The three tests must be equal
print(
    "PartialSpearmanX0Y=",
    ot.HypothesisTest.PartialSpearman(sampleX, sampleY, selection, 0.10),
)
print("SpearmanX0Y=", ot.HypothesisTest.Spearman(sampleX0, sampleY, 0.10))
print("FullSpearmanX0Y=", ot.HypothesisTest.FullSpearman(sampleX0, sampleY, 0.10))

print(
    "PartialSpearmanXY=",
    ot.HypothesisTest.PartialSpearman(sampleX, sampleY, selection, 0.10),
)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

stratas = 3
for dimension in range(1, 4):
    f = ot.NormInfEnumerateFunction(dimension)
    size = f.getStrataCumulatedCardinal(stratas)
    print("Strata", stratas, "cardinal=", f.getStrataCardinal(stratas))
    print("First", size, "values for dimension", dimension)
    for i in range(size):
        print("i=", i, "f(i)=", f(i))

# without bounds
f = ot.NormInfEnumerateFunction(2)
for index in range(25):
    m = f(index)
    index_inv = f.inverse(m)
    print("index=", index, repr(m))
    assert index == index_inv, "wrong inverse"
# with bounds
f = ot.NormInfEnumerateFunction(2)
f.setUpperBound([int(1e8), 3])
for index in range(25):
    m = f(index)
    index_inv = f.inverse(m)
    print("index=", index, repr(m))
    assert m[1] <= 3, "wrong bound"
    assert index == index_inv, "wrong inverse"
f = ot.NormInfEnumerateFunction(2)
f.setUpperBound([3, int(1e8)])
for index in range(25):
    m = f(index)
    index_inv = f.inverse(m)
    print("index=", index, repr(m), index_inv)
    assert m[0] <= 3, "wrong bound"
    assert index == index_inv, "wrong inverse"
#
print("Test getMarginal() from indices")
f = ot.NormInfEnumerateFunction(5)
marginalf = f.getMarginal([0, 3, 4])
for index in range(10):
    m = marginalf(index)
    index_inv = marginalf.inverse(m)
    print("index=", index, repr(m), index_inv)
#
print("Test getMarginal() from a single integer")
f = ot.NormInfEnumerateFunction(5)
marginalf = f.getMarginal(3)
for index in range(10):
    m = marginalf(index)
    index_inv = marginalf.inverse(m)
    print("index=", index, repr(m), index_inv)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


# We create a numerical math function
func = ot.SymbolicFunction(("E", "F", "L", "I"), ("-F*L^3/(3.*E*I)",))

dim = func.getInputDimension()

# We create a normal distribution point of dimension 1
# E, F, L, I
mean = [50.0, 1.0, 10.0, 5.0]
sigma = [1.0] * dim
dist = ot.Normal(mean, sigma)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(dist)

# We create a composite random vector
output = ot.CompositeRandomVector(func, vect)

# We create an Event from this RandomVector
event = ot.ThresholdEvent(output, ot.Less(), -3.0)

# We create a Monte Carlo algorithm
# event.isComposite() --> automatically add a MonteCarloExperiment
algo = ot.ProbabilitySimulationAlgorithm(event)
algo.setMaximumOuterSampling(500)
algo.setBlockSize(10)
algo.setKeepSample(True)

# Perform the simulation
algo.run()

# input / output sampls
input_sample = algo.getInputSample()
output_sample = algo.getInputSample()

assert len(input_sample) == 680
ott.assert_almost_equal(
    algo.getResult().getProbabilityEstimate(), 0.13970588235294112, 1e-3, 1e-3
)



import openturns as ot
import openturns.experimental as otexp
import openturns.viewer as otv

# Graph
g = ot.Graph("Fejer quadratures", "nodes", "")
g.setIntegerYTick(True)
discretization = [15]

# Fejer type 1
experiment = otexp.FejerExperiment(discretization, otexp.FejerExperiment.FEJERTYPE1)
nodes, weights = experiment.generateWithWeights()
lower = ot.Cloud(nodes, ot.Sample(nodes.getSize(), [0.0]))
lower.setColor("green")
lower.setPointStyle("circle")
g.add(lower)

# Fejer type 2
experiment = otexp.FejerExperiment(discretization, otexp.FejerExperiment.FEJERTYPE2)
nodes, weights = experiment.generateWithWeights()
lower = ot.Cloud(nodes, ot.Sample(nodes.getSize(), [1.0]))
lower.setColor("red")
lower.setPointStyle("square")
g.add(lower)

# Clenshaw-Curtis
experiment = otexp.FejerExperiment(discretization, otexp.FejerExperiment.CLENSHAWCURTIS)
nodes, weights = experiment.generateWithWeights()
lower = ot.Cloud(nodes, ot.Sample(nodes.getSize(), [2.0]))
lower.setColor("magenta")
lower.setPointStyle("plus")
g.add(lower)

g.setTitle(r"Fejer experiment example")

g.setLegends(["Fejer_1", "Fejer_2", "Clenshaw-Curtis"])
g.setLegendPosition("upper right")

otv.View(g, figure_kw={"figsize": (8, 4)}, add_legend=True)



"""
Create the distribution of the maximum of distributions
=======================================================
"""

# %%
# Context
# -------
#
# In this example, we define how to build a distribution defined as the maximum of some other ones.
#
# We detail some interesting cases:
#
# - Case 1: The maximum of correlated scalar distributions,
# - Case 2: The maximum of independent scalar distributions,
# - Case 3: The maximum of independent and identically distributed scalar distributions.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Case 1: The maximum correlated scalar distributions
# ---------------------------------------------------
#
# We consider :math:`d=3` scalar distributions :math:`X_i` that form a random vector
# :math:`(X_1, X_2, X_3)` which copula is denoted by :math:`C`.
#
# We assume that:
#
# .. math::
#    X_1 \sim \cU(2.5, 3.5)\\
#    X_2 \sim \mbox{Log}\cU(1.0, 1.2)\\
#    X_3 \sim \cT(2.0, 3.0, 4.0)\\
#    C \sim \mbox{NormalCopula}(\mat{M})
#
# where :math:`\mat{M}` is a correlation matix.
#
# We want to define the distribution of:
#
# .. math::
#    Y_1 = \max(X_1, X_2, X_3).

# %%
# We create the distribution of :math:`(X_1, X_2, X_3)`:
dist_X1 = ot.Uniform(2.5, 3.5)
dist_X2 = ot.LogUniform(1.0, 1.2)
dist_X3 = ot.Triangular(2.0, 3.0, 4.0)
coll = [dist_X1, dist_X2, dist_X3]
cor_mat = ot.CorrelationMatrix([[1.0, -0.9, 0.0], [-0.9, 1.0, 0.3], [0.0, 0.3, 1.0]])
cop = ot.NormalCopula(cor_mat)
dist_X = ot.JointDistribution(coll, cop)

# %%
# We create the distribution of :math:`Y_1`:
dist_Y_1 = ot.MaximumDistribution(dist_X)
print(dist_Y_1)

# %%
# We draw its PDF:
graph = dist_Y_1.drawPDF()
graph.setTitle(r" Distribution of $Y_1 = max(X_1, X_2, X_3)$")
graph.setXTitle(r"$Y_1$")
graph.setLegendPosition("")
view = otv.View(graph)

# %%
# Case 2: The maximum of independent scalar distributions
# -------------------------------------------------------
#
# Here, we define :math:`(\tilde{X}_1,\tilde{X}_2, \tilde{X}_3)` the random vector where the
# :math:`\tilde{X}_i` are distributed as :math:`X_i` and are independent.
#
# We want to define the distribution of:
#
# .. math::
#    Y_2 = \max(\tilde{X}_1,\tilde{X}_2, \tilde{X}_3).

# %%
# We create the distribution of :math:`Y_2`:
dist_Y_2 = ot.MaximumDistribution(coll)

# %%
# We could have written:
dist_X_tilde = ot.JointDistribution(coll)
dist_Y_2 = ot.MaximumDistribution(dist_X_tilde)

# %%
# We draw its PDF:
graph = dist_Y_2.drawPDF()
graph.setTitle(r" Distribution of $Y_2 = max(\tilde{X}_1,\tilde{X}_2, \tilde{X}_3)$")
graph.setXTitle(r"$Y_2$")
graph.setLegendPosition("")
view = otv.View(graph)

# %%
# Case 3: The maximum of a independent and identically distributed scalar distributions
# -------------------------------------------------------------------------------------
#
# Here we suppose that :math:`X_4` and :math:`X_5` are independent and identically distributed
# as :math:`X_1`.
#
# We want to define the distribution of:
#
# .. math::
#    Y_3 = \max(X_1, X_4, X_5).

# %%
# We create the distribution of :math:`Y_3`:
dist_Y_3 = ot.MaximumDistribution(dist_X1, 3)

# %%
# We draw its PDF:
graph = dist_Y_3.drawPDF()
graph.setTitle(r" Distribution of $Y_3 = max(X_1, X_4, X_5)$")
graph.setXTitle(r"$Y_3$")
graph.setLegendPosition("")
view = otv.View(graph)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# distance function from an Interval of dimension 0
singleton = ot.Interval()
assert singleton.computeDistance([]) == 0.0
assert ot.DistanceToDomainFunction(singleton)([]) == ot.Point([0.0])

# distance function from an Interval
box = ot.Interval([1.2, 0.7], [2.0, 1.0])
distance = ot.DistanceToDomainFunction(box)

point1 = ot.Point([1.5, 0.8])
assert box.computeDistance(point1) == 0.0
distance_to_point1 = distance(point1)
assert distance_to_point1 == ot.Point([0.0])

point2 = ot.Point([1.1, 0.8])
ott.assert_almost_equal(box.computeDistance(point2), 0.1, 1e-12, 0.0)
distance_to_point2 = distance(point2)
ott.assert_almost_equal(distance_to_point2, ot.Point([0.1]), 1e-12, 0.0)

sample = ot.Sample([point1, point2])
ott.assert_almost_equal(
    box.computeDistance(sample), ot.Sample([[0.0], [0.1]]), 1e-12, 0.0
)
ott.assert_almost_equal(distance(sample), ot.Sample([[0.0], [0.1]]), 1e-12, 0.0)

# distance function from a DomainUnion of Intervals
interval1 = ot.Interval(-0.5, 0.0)  # [-0.5, 0]
interval2 = ot.Interval(0.7, 1.3)  # [0.7, 1.3]
interval3 = ot.Interval(0.9, 1.0)
interval3.setFiniteUpperBound([False])  # [0.9, +infinity)

small_union = ot.DomainUnion([interval1, interval2])  # [-0.5, 0] U [0.7, 1.3]
# [-0.5, 0] U [0.7, +infinity)
big_union = ot.DomainUnion([small_union, interval3])

sample_dim1 = ot.Sample([[0.1], [1.4]])
ott.assert_almost_equal(
    small_union.computeDistance(sample_dim1), [[0.1]] * 2, 1e-12, 0.0
)
ott.assert_almost_equal(
    big_union.computeDistance(sample_dim1), [[0.1], [0.0]], 1e-12, 0.0
)

distance_to_small_union = ot.DistanceToDomainFunction(small_union)
ott.assert_almost_equal(distance_to_small_union(sample_dim1), [[0.1]] * 2, 1e-12, 0.0)
distance_to_big_union = ot.DistanceToDomainFunction(big_union)
ott.assert_almost_equal(distance_to_big_union(sample_dim1), [[0.1], [0.0]], 1e-12, 0.0)



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott

f = ot.SymbolicFunction(
    ot.Description.BuildDefault(5, "x"),
    [
        "sin(x0) + x1 * cos(x2) / exp(x3)",
        "-2.0 * x0 + x1 * x2^2 + cos(x3)",
        "x0 / (abs(x1) * x2^2 + x3 + 1.0)",
    ],
)
pset = [3, 1]  # x3, x1
parametersSet = True
x = [1.0, 2.0, 0.0]  # x0, x2, x4

# with reference point
referencePoint = [0.85] * 2  # x3, x1
g2 = ot.ParametricFunction(f, pset, referencePoint, parametersSet)
print("g2=", g2)
print("g2(x)=", g2(x))

# check output description
outputDesc = ot.Description.BuildDefault(3, "g2_")
g2.setOutputDescription(outputDesc)
X = ot.Normal(3).getSample(10)
Y = g2(X)
assert Y.getDescription() == outputDesc, "wrong output description"

# with complementary parameter set
referencePoint = [1.0, 2.0, 0.0]  # x0, x2, x4
x = [0.85] * 2  # x3, x1
parametersSet = False
g3 = ot.ParametricFunction(f, pset, referencePoint, parametersSet)
print("g3=", g3)
print("g3(x)=", g3(x))

# From ticket #1092
modelFunc = ot.SymbolicFunction(["s", "r", "c"], ["r + c"])
s = [1.0]
r = 1e9
c = 1e8
theta = ot.Point([r, c])
calibratedIndices = [1, 2]
mycf = ot.ParametricFunction(modelFunc, calibratedIndices, theta)

print(modelFunc.gradient([s[0], r, c]).transpose())
print(mycf.parameterGradient(s).transpose())

# Check if parametric functions and memoize functions work well together

n_calls = 0


def py_f(X):
    global n_calls
    n_calls += 1
    return X


ot_f = ot.MemoizeFunction(ot.PythonFunction(3, 3, py_f))
param_f = ot.ParametricFunction(ot_f, [0, 1], [1.0, 2.0])

x = [3.0]
y = [1.5]
n_calls_0 = ot_f.getEvaluationCallsNumber()
par_grad = param_f.parameterGradient(x)
n_calls_1 = ot_f.getEvaluationCallsNumber()
assert (
    n_calls_1 - n_calls_0 == 4
), "Expected n_calls_1 - n_calls_0 == 4, here n_calls_1 - n_calls_0 == " + str(
    n_calls_1 - n_calls_0
)
assert n_calls == 4, "Expected n_calls == 4, here n_calls == " + str(n_calls)

n_calls = 0
n_calls_0 = ot_f.getEvaluationCallsNumber()
f_grad = param_f.gradient(y)
n_calls_1 = ot_f.getEvaluationCallsNumber()
assert (
    n_calls_1 - n_calls_0 == 2
), "Expected n_calls_1 - n_calls_0 == 2, here n_calls_1 - n_calls_0 == " + str(
    n_calls_1 - n_calls_0
)
assert n_calls == 2, "Expected n_calls == 2, here n_calls == " + str(n_calls)

eps = 1e-7
gr_f = ot.NonCenteredFiniteDifferenceGradient(eps, ot_f.getEvaluation())
ot_f.setGradient(gr_f)
n_calls = 0
param_f = ot.ParametricFunction(ot_f, [0, 1], [1.0, 2.0])

x = [3.5]
y = [2.5]
n_calls_0 = ot_f.getEvaluationCallsNumber()
par_grad = param_f.parameterGradient(x)
n_calls_1 = ot_f.getEvaluationCallsNumber()
assert (
    n_calls_1 - n_calls_0 == 3
), "Expected n_calls_1 - n_calls_0 == 3, here n_calls_1 - n_calls_0 == " + str(
    n_calls_1 - n_calls_0
)
assert n_calls == 3, "Expected n_calls == 3, here n_calls == " + str(n_calls)

n_calls = 0
n_calls_0 = ot_f.getEvaluationCallsNumber()
f_grad = param_f.gradient(y)
n_calls_1 = ot_f.getEvaluationCallsNumber()
assert (
    n_calls_1 - n_calls_0 == 2
), "Expected n_calls_1 - n_calls_0 == 2, here n_calls_1 - n_calls_0 == " + str(
    n_calls_1 - n_calls_0
)
assert n_calls == 2, "Expected n_calls == 2, here n_calls == " + str(n_calls)

# check parameters name are shown
html_str = g2._repr_html_()
assert "[x3 : 0.85, x1 : 0.85]" in html_str, "no params"

# check parameters order
f1 = ot.SymbolicFunction(["x1", "x2", "x3"], ["2*x1+3*x2+5*x3"])
f1.setGradient(ot.NonCenteredFiniteDifferenceGradient([1e-5] * 3, f1.getEvaluation()))
f2 = ot.ParametricFunction(f1, [1, 0], [4.0, 5.0])
pgrad = f2.parameterGradient([7.0])
ott.assert_almost_equal(pgrad[0, 0], 3.0)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.Exponential(2.5, -1.3)
size = 10000
sample = distribution.getSample(size)
factory = ot.ExponentialFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedExponential = factory.buildAsExponential(sample)
print("Exponential          =", distribution)
print("Estimated exponential=", estimatedExponential)
estimatedExponential = factory.buildAsExponential()
print("Default exponential=", estimatedExponential)
estimatedExponential = factory.buildAsExponential(distribution.getParameter())
print("Exponential from parameters=", estimatedExponential)
